/*
  Copyright 2004 Michalis Kamburelis.

  This file is part of "Jamy & Nory".

  "Jamy & Nory" is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  "Jamy & Nory" is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with "Jamy & Nory"; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package jamy_i_nory.client;

import java.io.*;
import java.nio.channels.*;
import java.util.*;
import java.net.*;
import jamy_i_nory.common.*;
import jamy_i_nory.common.iosystem.*;

/** Oglna klasa klienta, uruchamiaj po prostu robic new Client z odpowiednimi
   parametrami, tzn. uruchamiajc konstruktor. Po zakoczeniu dziaania 
   konstruktora nie ma sensu przechowywa gdzie instancji tej klasy. 
   
   <p>TODO: ta klasa nie powinna robi nigdzie System.exit (eby by uyteczna
   jako applet). Sprawdzi czy moe wypisywa co na System.err (bo tutaj i 
   w StringSocket czasami co wypisujemy), prawdopodobnie nie, wic to te trzeba
   poprawi. Zreszt nawet gdyby nie byo trzeba to i tak powinno si tak zrobi.
   Wszystko powinno i przez io. Zrobienie tego wymaga jednak najpierw
   zrobienia innego TO-DO:
   "gdy interfejs w Swingu si koczy to user nie ma szans przeczyta ostatnich
    rzeczy jakie mu wypisze (w rodzaju "Podae ze haso")" */
public class Client {

  private String describeSocket(Socket s)
  {
    return "Lokalny adres : " +s.getLocalAddress() 
             +", port : " +s.getLocalPort() +Utils.nl+
           "Zdalny adres : " +s.getInetAddress() 
             +", port : " +s.getPort() + Utils.nl;
  }
 
  final private IOSystem io;
  final private StringSocket sSocket;
  
  static class ConnectionClosedException extends Exception {};
  
  /** Zwraca sSocket.read(). Ale najpierw sprawdza czy sSocket.isConnected
     i jeli nie - rzuca ConnectionClosedException. W ten sposb zamiast
     nieustannie testowa sSocket.isConnected moesz tylko apa
     wyjtek ConnectionClosedException - to jest wygodniejsze. */
  private String sSocketRead() 
  throws ConnectionClosedException
  {
    if (!sSocket.isConnected()) throw new ConnectionClosedException();
    return sSocket.read();
  }
  
  /** Podobnie jak sSocketRead - robi sSocket.write(value) sprawdzajc 
     uprzednio czy sSocket.isConnected (i jeli nie to rzuca 
     ConnectionClosedException). */
  private void sSocketWrite(String value) 
  throws ConnectionClosedException
  {
    if (!sSocket.isConnected()) throw new ConnectionClosedException();
    sSocket.write(value);
  }
  
  /** Wywouje sSocketRead tak dugo a zwrci co != null. Wtedy robi io.print
     tego co zwrcio sSocketRead. 
     
     <p>Czsami mona tego uy bo np. zawsze serwer odpowiada co na wysan 
     do niego komend i zawsze po podczeniu si do gry wysya powitalny 
     message.
     
     <p>Tym niemniej uywanie poniszej metody jest zupenie opcjonalne -    
     ale sprawia e interfejs konsolowy wyglda lepiej. W ten sposb moemy 
     do pewnego stopnia unikn wypisywania w czasie lineReading() 
     co wyglda paskudnie na interfejsie konsolowym. Oczywicie to nie jest nic
     pewnego bo de facto nie wiemy czy odczytany komunikat to jest odpowied na
     nasz komend (by moe jest to jaki wczeniejszy komunikat) - mgbym to
     sprawdza wprowadzajc znaczniki numerw komend ale, prawd mwic, nie
     warto si mczy; jak powiedziaem, jest to potrzebne tylko po to aby
     interfejs konsolowy wyglda lepiej. */
  private void waitForMessageAndPrint()
  throws ConnectionClosedException
  {
    /* TODO:May sleep eby busy-waiting nie bolao tak bardzo */
    String serverAnswer;
    while ((serverAnswer = sSocketRead()) == null) ;
    io.print(serverAnswer);
  }
  
  /** Uses {@link #io} to get required information about new player, 
     then sends it over sSocket (one string = playerName,
     second string = raceNumber on Race.playerRaces,
     third string = password). Then waits and prints first starting message 
     from server (using  waitForMessageAndPrint) */
  private void newPlayer()
  throws ConnectionClosedException
  { 
    io.println("----------------------------------------");
    io.println("*** Tworzenie nowego gracza ***");

    String playerName = io.readString("Imi:", false);

    io.println("  [-1] Wznw gr graczem zamroonym na serwerze");
    
    /* TODO: chwilowa strata funkcjonalnoci.
       Klient nie moe siga do pakietu game, bo przecie klient w ogle
       nic nie robi z mechanik gry -- ponisze instrukcje byy tylko po
       to eby pokaza userowi adne stringi.
       Serwer powinien przysya klientowi list dostpnych ras.
       
    for (int i=0; i<Race.playerRaces.length; i++)
      io.println("  [" +i +"] " +Race.playerRaces[i].toDetailString());
    int raceNum = io.readInt("Wybierz ras:" , -1, Race.playerRaces.length-1);
    */
    io.print(
      "  [0] Czowiek [sia: 7+1k5; zrczno: 7+1k5; magia: 5+1k5; max ycie: 17+1k5; inteligencja: 7+1k5; sia ataku bez broni: 1]" +Utils.nl+
      "  [1] Elf [sia: 5+1k5; zrczno: 9+1k5; magia: 7+1k5; max ycie: 15+1k5; inteligencja: 7+1k5; sia ataku bez broni: 1]" +Utils.nl+
      "  [2] Krasnolud [sia: 9+1k5; zrczno: 5+1k5; magia: 2+1k5; max ycie: 19+1k5; inteligencja: 7+1k5; sia ataku bez broni: 1]" +Utils.nl
      );
    int raceNum = io.readInt("Wybierz ras:" , -1, 2);
    
    String password;
    if (raceNum != -1)
      password = io.readString("Podaj haso ktre bdzie zabezpieczao" 
        +" Twoj posta (aby nikt poza Tob nie mg ni gra):", false);
    else 
      password = io.readString("Podaj haso ktre zabezpiecza posta:", false);

    sSocketWrite(playerName);
    sSocketWrite("" + raceNum);
    sSocketWrite(password);
    waitForMessageAndPrint();
  }  
  
  public Client(String serverHost, int serverPort, IOSystem aIO)
  throws IOException
  {
    /* init socketChannel */    
    SocketChannel socketChannel = null;
    try {      
      socketChannel = SocketChannel.open(new InetSocketAddress(
        InetAddress.getByName(serverHost), serverPort));
    } catch (UnknownHostException e) {
      System.err.println("Nieznany serwer: " +e);      
      System.exit(1);
    } catch (IOException e) {
      System.err.println("Bd podczas tworzenia gniazda i czenia z serwerem: " +e); 
      System.exit(1);
    }

    /* init io */
    io = aIO;

    try { 
      io.print("Poczenie z serwerem ustanowione." +Utils.nl +
               describeSocket(socketChannel.socket()));

      sSocket = new StringSocket(socketChannel);
      try {
               
        newPlayer();
       
        io.lineBegin();
        try {
          while (true)
          {
            /* Check for messages from server. Wszystko co przyle serwer 
               po prostu wypisz.  */
            String serverMessage;
            if ((serverMessage = sSocketRead()) != null)
              io.print(serverMessage);

            /* check user input */
            if (io.lineReady())
            {
              String sendString = io.lineEnd();
              /* exit cleanly when we ran out of input */
              if (sendString == null) {
                io.println("Koniec (stdin zakoczone).");
                break;
              }
              sSocketWrite(sendString);
              waitForMessageAndPrint();
              
              io.lineBegin();
            }
          }
        } finally { io.lineEndCancel(); }
      
      } catch (ConnectionClosedException e) { /* silence exception */ }
              
      sSocket.close();

      io.println("Poczenie z serwerem zakoczone.");
    } finally { io.dispose(); }    
  }
}

/*
 * $Log: Client.java,v $
 * Revision 1.17  2004/02/19 21:54:16  michalis
 * + klasa ClientApplet ostatecznie zrobiona - dziaa !
 * * useStyles wycignite jako parametr konstruktorw IOSwing
 * + JamyUtils z pomocniczymi rzeczami gromadzcymi parametry do interfejsw,
 *   teraz i serwer i klient i single maj peen zestaw 3 parametrw --console, --simple-swing, --stylish-swing
 * * odpowiednie uaktualnienia w dokumentacji
 *
 * Revision 1.16  2004/02/19 15:30:25  michalis
 * * Creature.attackPlayers/Creatures -> SimpleMindedCreature
 * + NonHostileCreature
 * + ElfFairy, ElfFairyWhistle
 * + wanderingFactor, teraz prawie wszystkie creatures wasaj si po wiecie
 *
 * Revision 1.15  2004/02/18 14:37:20  michalis
 * * pl stringi
 * * --help output changed (to reflect that those classes will be run by bin/scripts)
 * preparations for ClientApplet:
 * * Client rozdzielony na Client i ClientStandalone
 * * znaczna cz pakietu common przeniesiona do nowego pakietu game
 * X* make jamy_i_nory_client_applet.jar
 *
 * Revision 1.14  2004/02/10 02:17:57  michalis
 * * lotsa comments
 *
 * Revision 1.13  2004/02/10 00:39:40  michalis
 * * po kilku bezproduktywnych commitach (jakie reorganizacje ...) co konkretnego:
 *   Zamroony na serwerze gracz jest chroniony hasem.
 *
 * Revision 1.12  2004/02/09 23:08:47  michalis
 * + GPL license clause i copyrighty (moje i gdzieniegdzie Michaa, sorry Rafa
 *   ale naprawd nic nie zostao) do wszystkich *.java
 *
 * Revision 1.11  2004/02/06 06:20:58  michalis
 * + PlayerConnection: dodatkowy konstruktor
 * + InactivePlayersList
 * * rozszerzona funkcjonalnoc Connection (musi umie wywoa oba konstruktory
 *   PlayerConnection)
 *   Dziaa ! Zamraanie i odmraanie graczy zrobione !
 *
 * Revision 1.10  2004/02/05 23:27:36  michalis
 * * jeszcze mae poprawki w kwestii kiedy pisa TO-DO a kiedy to samo bez mylnika
 * * IOSwing konstruktor: parametr windowTitle
 * + Server obsuguje parametr --swing, ma pole IOSystem
 * + Utils.collectionToMultiLineString(Collection,String)
 * * Server mona ju kontrolowa interfejsem IOSystem, polecenia "x", "h", "p", "c"
 * + World.allCreatures/PlayersToString,
 * * UserCommand. "debug players/creatures" uywaj World.allCreatures/PlayersToString
 * * Poprawka w Server.ServerListener.run przy sleep
 *
 * Revision 1.9  2004/02/05 20:28:18  michalis
 * * Klasy IOSystem/Std/Swing przeniesione do pakietu common.iosystem.
 *   Byy uywane take przez single/ wic ich dotychczasowa lokalizacja
 *   w client/ bya za. Ponadto zaraz bd uywane take przez server/.
 *
 * Revision 1.8  2004/02/05 20:05:23  michalis
 * * Zrobiem eby w logach CVSa zapisanych w plikach byo wszdzie TO-DO
 *   z mylnikiem w rodku. Wprawdzie nieznacznie rozsynchronizowuje to
 *   zapisane w plikach logi z wynikiem 'cvs log' ale za to grep TO-DO
 *   (gdzie w wyrazie TO-DO naley usun mylnik) dziaa jak trzeba.
 *
 * Revision 1.7  2004/02/05 19:26:12  michalis
 * * docs corrected
 *
 * Revision 1.6  2004/02/04 18:56:40  michalis
 * + some improvements in Client (waitForMessageAndPrint, sSocketRead, sScoketWrite)
 * + Player.followLink
 * * UserCommand.execute does not return String (it returns void).
 *   It should simply append something to player.getMessage().
 *   Wiele uproszcze zapisu w UserCommand - np. komenda "idz" jest teraz
 *   krciutka i wszystkim zajmuje si Player.followLink i Player.setNowLocation.
 * + PlayerConnection.pushMessage, * PlayerConnection.dead slightly improved
 * - Server.sendMessage, - Server.broadcastMessage (useless - we use only
 *   sendPlayersMessages and PlayerConnection.pushMessage)
 *
 * Revision 1.5  2004/02/01 01:16:53  michalis
 * * small docs improvements
 * - Client testing message ("Enter lines ...") removed
 *
 * Revision 1.4  2004/02/01 00:25:30  michalis
 * * Client wypisuje otrzymane od serwera stringi wprost (bez jakiego prefixu
 *   "Serwer przysa wiadomo" i bez dodawania newline'a na kocu)
 * * PlayerConnection: po wysaniu "Umierasz." dodaje newline
 *
 * Revision 1.3  2004/02/01 00:08:51  michalis
 * + Being.DeadAnswer, dead() returns DeadAnswer
 * - Player/Creature.dead
 * * Player/Creature.setNowLocation: remove object from world.players/creatures list
 * * World.action implemented cleaner
 * * Creature.attack dostaje parametr PlayersMessages
 * + Creature.ElfKiller, dodany do World do lokacji "Jama"
 * + PlayersMessages.append
 * + PlayerConnection.server, + Connection.createPlayer "server" parameter
 * + gdy gracz umiera to jest komunikat dla System.out serwera, komunikat "Umierasz"
 *   dla umierajcego gracza i komunikat "kto-tam umiera" dla pozostaych graczy
 *   w tej lokacji
 * * another bugfix for checking dead state of players in server.
 *   Wszystko przez to e na razie mam doc niewygodne metody testowania tego kodu
 *   skoro gracze nie mog si rusza w wiecie. No, ale myl e obecnie
 *   wszystko ju jest jak trzeba - wszystkie elementy kodu zostay
 *   przetestowane dziki ElfKillerowi.
 *
 * Revision 1.2  2004/01/31 21:40:42  michalis
 * + IOSystem and subclasses: method lineEndCancel
 * * JamySingle and Client use lineEndCancel to avoid some bug
 *
 * Revision 1.1  2004/01/31 20:47:46  michalis
 * * JamySingle parameter console changed to --console
 * * UserCommand moved from tests/ to common/
 * * UserCommand small changes (very small doc updates, some things made public
 *   to be available from outside of package jamy_i_nory.common)
 * * tests/SimpleClient.java moved to client/Client.java
 * * tests/local_client.sh moved to client/ and modified
 * * Server: StringBuffer[] changed to PlayerMessages everywhere
 *
 * Logs as tests/SimpleClient.java:
 * Revision 1.9  2004/01/31 06:02:12  michalis
 * + Server.world
 * + SimpleClient.newPlayer
 * * Idea of Connection changed : now it does not inherit from StringSocket.
 *   It just holds stringSocket inside, and can create PlayerConnection.
 * + PlayerConnection : inherits from Player, contains StringSocket.
 *   (This is what was eariler meant to be in Connection)
 * * Server.users split to connections and world.players()
 * + Server adds ready connections to world.players() using Connection.createPlayer
 * - Server.findById (it was just a TO-DO) removed,
 *   now it can be implemented using IdedList.byId
 * + important comments at world.action
 * * StringSocket.read bug (when receiving two messages *fast*) fixed
 * + Utils.InternalError(String, Throwable)
 * * some Server docs improved
 *
 * Revision 1.8  2004/01/30 00:59:46  michalis
 * * StringSocket.read small fix
 * + Server.broadcastMessage
 * - Removed some TO-DO in Server by Rafa; jeli dobrze je rozumiem to wykonaem je
 *   robic klas StringSocket
 * + Server wysya wiadomo do wszystkich klientw gdy podcza si nowy klient
 *   (to tylko testowe, w kocowej grze to nie ma by widoczne dla wszystkich)
 * + SimpleClient odbiera wiadomoci od serwera i wywietla; w ten sposb Server
 *   i SimpleClient robi pen komunikacj - obaj mog co wysya,
 *   obaj mog co odbiera
 *
 * Revision 1.7  2004/01/30 00:30:27  michalis
 * * SimpleServer renamed to Server
 *
 * Revision 1.6  2004/01/30 00:12:03  michalis
 * + SimpleServer uses Utils.parseCommandLine, params --help and --port implemented
 * + Utils.DEFAULT_JAMY_PORT
 * * SimpleClient: small improvement to message printed by --help
 * * small improvement to local_server.sh and server.sh
 *
 * Revision 1.5  2004/01/29 23:42:45  michalis
 * * SimpleClient uses non-blocking input from IOSystem (lineBegin/End, as in JamySingle)
 * * SimpleClient uses Swing interface by default
 * + Utils.parseCommandLine
 * + SimpleClient uses command-line parsing, params --help, --port, --console
 *
 * Revision 1.4  2004/01/29 20:16:33  michalis
 * * SimpleClient uses StringSocket
 *
 * Revision 1.3  2004/01/29 04:00:57  michalis
 * + Connection.toString
 * + SimpleClient wysya bajty (ze Stringa)
 * + SimpleServer odbiera bajty i wypisuje na konsol (jako String) - co i od kogo dosta
 * * Now SimpleServer can be stopped only by C-c (i.e., killing the process).
 *   Functionality to close server "gently" will be restored soon.
 *
 * Revision 1.2  2004/01/27 01:03:29  michalis
 * * SimpleClient - some fixes: nices messages for user when something
 *   (e.g. connection) fails, more consistent use of IOSystem.
 *
 * Revision 1.1  2004/01/26 20:56:47  michalis
 * + SimpleClient
 * * SimpleServer made public (so java jamy_i_nory.server.SimpleServer works)
 * * SimpleServer fixed (users were never created !)
 *
 */
