/*
  Copyright 2004 Michalis Kamburelis.

  This file is part of "Jamy & Nory".

  "Jamy & Nory" is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  "Jamy & Nory" is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with "Jamy & Nory"; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package jamy_i_nory.client;

import java.io.*;
import java.nio.channels.*;
import jamy_i_nory.common.*;
import jamy_i_nory.common.iosystem.*;

/** This class provides method {@link #main} to start {@link Client} 
   from command-line. */
public class ClientStandalone {

  static public void main(String[] argv)
  throws IOException
  {
    class OptionProcesser implements Utils.CommandLineOptionProcesser {  
      String paramServerHost; 
      int paramServerPort = JamyUtils.DEFAULT_JAMY_PORT;
      final int DEFAULT_IO = JamyUtils.IO_SIMPLE_SWING;
      int paramIO = DEFAULT_IO;

      public void processCommandLineOption(int optionNum, String[] optionArgs)
      {
        switch (optionNum)
        {
          case -1:
            if (paramServerHost != null) 
            {
              System.err.println("Excessive parameter \"" +optionArgs[0] +"\""); 
              System.exit(1); 
            } else
              paramServerHost = optionArgs[0];
            break;

          case 0: 
            System.out.print(
              "Usage: jamy_client <server-host-name-or-ip> [OPTIONS...]" +Utils.nl +
              "  where available options are" +Utils.nl +
             /*01234567890123456789*/
              "  --help            Show this help" +Utils.nl +
              "  --port PORT       Connect with server on PORT (default port is " 
                +JamyUtils.DEFAULT_JAMY_PORT+ ")" +Utils.nl +
              JamyUtils.ioParamsHelp(DEFAULT_IO)
            );
            System.exit(0);

          case 1: 
            try {
              paramServerPort = Integer.parseInt(optionArgs[0]);
            } catch (NumberFormatException e) {
              System.err.println("Invalid number : " +e);
              System.exit(1);          
            }
            break;

          case 2: paramIO = JamyUtils.IO_CONSOLE; break;
          case 3: paramIO = JamyUtils.IO_SIMPLE_SWING; break;
          case 4: paramIO = JamyUtils.IO_STYLISH_SWING; break;
        }
      }
    };
    
    /* parse command-line */
    OptionProcesser optionProcesser = new OptionProcesser();
    Utils.parseCommandLine(argv, 
      new Utils.CommandLineOption[] {
        new Utils.CommandLineOption("help", 0),
        new Utils.CommandLineOption("port", 1),
        new Utils.CommandLineOption(JamyUtils.IO_PARAMS_NAMES[JamyUtils.IO_CONSOLE], 0),
        new Utils.CommandLineOption(JamyUtils.IO_PARAMS_NAMES[JamyUtils.IO_SIMPLE_SWING], 0),
        new Utils.CommandLineOption(JamyUtils.IO_PARAMS_NAMES[JamyUtils.IO_STYLISH_SWING], 0),
      }, optionProcesser);
    if (optionProcesser.paramServerHost == null) 
    {
      System.err.println("No server specified.");
      System.exit(1);
    }
  
    new Client(optionProcesser.paramServerHost, 
      optionProcesser.paramServerPort, 
      JamyUtils.newIO(optionProcesser.paramIO, "> ", "Jamy & Nory - klient"));
  }
}

/*
 * $Log: ClientStandalone.java,v $
 * Revision 1.2  2004/02/19 21:54:16  michalis
 * + klasa ClientApplet ostatecznie zrobiona - dziaa !
 * * useStyles wycignite jako parametr konstruktorw IOSwing
 * + JamyUtils z pomocniczymi rzeczami gromadzcymi parametry do interfejsw,
 *   teraz i serwer i klient i single maj peen zestaw 3 parametrw --console, --simple-swing, --stylish-swing
 * * odpowiednie uaktualnienia w dokumentacji
 *
 * Revision 1.1  2004/02/18 14:37:20  michalis
 * * pl stringi
 * * --help output changed (to reflect that those classes will be run by bin/scripts)
 * preparations for ClientApplet:
 * * Client rozdzielony na Client i ClientStandalone
 * * znaczna cz pakietu common przeniesiona do nowego pakietu game
 * X* make jamy_i_nory_client_applet.jar
 *
 */
