/*
  Copyright 2004 Michalis Kamburelis.

  This file is part of "Jamy & Nory".

  "Jamy & Nory" is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  "Jamy & Nory" is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with "Jamy & Nory"; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package jamy_i_nory.common;

import java.util.*;
import jamy_i_nory.common.iosystem.*; 

/** Some static utilities specific to Jamy&Nory. */
public class JamyUtils {

  /** Default port of our game's server. Constant useful for both
     client and server. */
  final static public int DEFAULT_JAMY_PORT = 1111; 
  
  final static public int IO_CONSOLE = 0;
  final static public int IO_SIMPLE_SWING = 1;
  final static public int IO_STYLISH_SWING = 2;
  final static public int IO_COUNT = 3;
  
  final static public String[] IO_PARAMS_NAMES = 
    { "console", "simple-swing", "stylish-swing" };
  
  /** Wielolinijkowy (zakoczony Utils.nl) help do wszystkich parametrw
     PARAM_IO_xxx. Parametr defaultIO jest oznaczony jako parametr
     domylny dla programu, to musi by jedna z tutejszych staych IO_xxx.
     Opisy parametrw s dosunite do kolumny 21. */
  final static public String ioParamsHelp(int defaultIO)
  {
    final String[] help = {
      "Use console (stdin/stdout) interface",
      "Use simple windowed (Swing) interface",
      "Use stylish windowed (Swing) interface",
    };
    
    StringBuffer result = new StringBuffer();
    for (int i = 0; i < IO_COUNT; i++)
    {
      result.append(Utils.rightPad("  --" +IO_PARAMS_NAMES[i], 20, ' ') +
        help[i] +(i == defaultIO ? " (DEFAULT)" : "") +Utils.nl);
    }
    
    return result.toString();
  }
  
  /** ioKind to jedna ze staych IO_xxx. Konstruuje odpowiedni instancj
     IOSystem. Parametr prompt bdzie przekazany tylko do IOStd,
     parametr windowTitle tylko do IOSwing. */
  final static public IOSystem newIO(int ioKind, String prompt, String windowTitle)
  {
    switch (ioKind)
    {
      case IO_CONSOLE: return new IOStd(prompt); 
      case IO_SIMPLE_SWING: return new IOSwing(windowTitle, false); 
      case IO_STYLISH_SWING: return new IOSwing(windowTitle, true);
      default: throw new InternalError("unknown ioKind");
    }
  }  
}

/*
 * $Log: JamyUtils.java,v $
 * Revision 1.1  2004/02/19 21:54:18  michalis
 * + klasa ClientApplet ostatecznie zrobiona - dziaa !
 * * useStyles wycignite jako parametr konstruktorw IOSwing
 * + JamyUtils z pomocniczymi rzeczami gromadzcymi parametry do interfejsw,
 *   teraz i serwer i klient i single maj peen zestaw 3 parametrw --console, --simple-swing, --stylish-swing
 * * odpowiednie uaktualnienia w dokumentacji
 *
 */
