/*
  Copyright 2004 Michalis Kamburelis.

  This file is part of "Jamy & Nory".

  "Jamy & Nory" is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  "Jamy & Nory" is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with "Jamy & Nory"; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package jamy_i_nory.game;

/** Klasa pomocnicza to wyraania nastawienia jednego Being do drugiego.
   W tym momencie nastawienie wyraamy jako int a ta klasa po prostu
   pomaga nam je przetwarza - by moe z czasem to si zmieni,
   gdy nastawienie jednego Being do drugiego bedziemy wyraali w sposb
   bardziej skomplikowany.  
   
   <p>"Nasz int" w dokumentacji tej klasy oznacza int ktry
   moe przyjmowa jako wartoci tylko zdefiniowane w tej klasie
   stae NEUTRAL / HOSTILE / UNKNOWN. 
   
   <p>TODO: probably later this class will be moved to package-only
   access. */
public abstract class Attitude {

  final static public int NEUTRAL  = 0;
  final static public int HOSTILE  = 1;
  final static public int UNKNOWN  = 2;
  
  /** Konwersja naszego inta na String, String to jedno krtkie sowo 
     lowercase. */
  final static public String toStr[] = { "neutralne", "wrogie", "nieznane" };
  
  /** Zamie boolean na naszego inta. True daje HOSTILE,
     false NEUTRAL. A wic nigdy nie zwraca UNKNOWN. 
     
     
     <rafa>
     
     Proponowabym troch ostroniej na razie uywa deklaracji final 
     (i zamiast niej uzywac kompilacji deklaracji private i interfejsw get/set).
     Poza tym bybym za tym aby klasa ta bya abstrakcyjna - nie potrzebujemy tworzy 
     obiektw tej klasy, jest nam potrzebna jedynia jako narzedzie. 
     Metoda 'isHostileToAttitude' nie musi byc ostateczna - jej podklasy mog przecie
     chcie j nadpisa kiedy "nastawienie jednego Being do drugiego bedziemy wyraali 
     w sposb bardziej skomplikowany" */
     
  static public int isHostileToAttitude(boolean isHostile)
  {
    if (isHostile) 
      return Attitude.HOSTILE; 
    else                         
      return Attitude.NEUTRAL;
  }
}

/* 
 * $Log: Attitude.java,v $
 * Revision 1.1  2004/02/18 14:37:28  michalis
 * * pl stringi
 * * --help output changed (to reflect that those classes will be run by bin/scripts)
 * preparations for ClientApplet:
 * * Client rozdzielony na Client i ClientStandalone
 * * znaczna cz pakietu common przeniesiona do nowego pakietu game
 * X* make jamy_i_nory_client_applet.jar
 *
 * Revision 1.7  2004/02/09 23:08:49  michalis
 * + GPL license clause i copyrighty (moje i gdzieniegdzie Michaa, sorry Rafa
 *   ale naprawd nic nie zostao) do wszystkich *.java
 *
 * Revision 1.6  2004/02/04 14:00:24  michalis
 * * Being.tellEveryoneInNowLocation -> sayEveryoneInNowLocation
 *   (mniej mylca nazwa)
 * * Server powiadamia wszystkich graczy w komnacie e pojawi si nowy gracz
 * * PlayersMessages.tellEveryoneInLocation - docs updated,
 *   implemented in a cleaner way
 * + MessageGenerator
 * + MessageFormatter
 * * Being.attitude changed to realAttitude and observedAttitude
 * * Many fixes to use MessageFormatter when updating PlayersMessages.
 *   Many TO-DOs closed.
 * * Attitude.toStr translated to polish
 * * Poprawiony bug w Being.dead - Micha, nie mona tworzy PlayersMessages
 *   tu przed usuniciem gracza z gry - w czasie uywania PlayersMessages
 *   world.players() nie mog by modyfikowane. Z drugiej strony, to jest moja
 *   wina e jeszcze tego nie mona zrobi (bo to *powinno* by moliwe tylko
 *   musz zapisa inteligentniej PlayersMessages). OK, wanie mi przyszo do
 *   gowy jak to zrobi bardzo atwo i bardzo elegancko (klasa PlayersMessages
 *   w ogle nie bdzie potrzebna i unikniemy przerzucania wszdzie parametrw
 *   playersMessages) i w nastpnym commicie to bdzie zrobione (gupi, gupi,
 *   gupi jestem e od pocztku tego tak nie zrobiem).
 * * OK, serwer skunk.one.pl nie odpowiada wic nie bd commitowa, od razu
 *   zrobi to o czym mwiem powyej:
 *   + Player.getMessage, Player.putMessage
 *   + Location.tellEveryone (3 wersje)
 *   - PlayersMessages (funkcjonalno przerzucona w caoci do obiektw Player i
 *     Location)
 *   * masa uproszcze kodu (m.in. nie musimy ju przerzuca wszdzie
 *     parametrw PlayersMessages)
 *   - klasa Being.DeadAnswer (useless, teraz Being.dead() znowu zwraca prostego
 *     booleana)
 *   * w ogle Being.dead() jest teraz o niebo prostsze.
 *   Skoczyem! PlayersMessages usunite razem z wieloma niewygodami.
 *
 * Revision 1.5  2004/01/16 19:39:22  michalis
 * * Poprawione polskie ogonki z Windows-1250 na ISO-8859-2. Rafa: no wiesz o co chodzi.
 *
 * Revision 1.4  2004/01/15 11:24:27  michalis
 * * tabs -> spaces (Rafal: nie uzywaj znakow TAB !)
 *
 * Revision 1.3  2004/01/15 00:30:04  bularaider
 * * static public int isHostileToAttitude(boolean isHostile)
 * * public abstract class Attitude,
 *
 * -> Bybym za tym aby klasa ta bya abstrakcyjna - nie potrzebujemy tworzy  obiektw tej klasy, jest nam potrzebna jedynia jako narzedzie.
 * -> Metoda 'isHostileToAttitude' nie musi byc ostateczna - jej podklasy mog przecie chcie j nadpisa kiedy "nastawienie jednego Being do drugiego bedziemy wyraali w sposb bardziej skomplikowany".
 *  -> Proponowabym troch ostroniej na razie uywa deklaracji final
 * (i zamiast niej uzywac kompilacji deklaracji private i interfejsw get/set).
 *
 * Revision 1.2  2004/01/12 02:20:04  michalis
 * * docs improved
 *
 * Revision 1.1  2004/01/12 02:02:44  michalis
 * + StatModifier.intelligence(), class Attitude, Being.attitude(),
 *   Being.toString(Being) extended to show attitude
 *
 */
