/*
  Copyright 2004 Michalis Kamburelis, Micha Saaban.

  This file is part of "Jamy & Nory".

  "Jamy & Nory" is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  "Jamy & Nory" is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with "Jamy & Nory"; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package jamy_i_nory.game;

import java.util.Random;
import jamy_i_nory.common.*;

/** Describes a way to compute a random value, known from many games:
   rzu {@link #dicesCount} razy {@link #diceFaces} - ciennymi kostkami
   (tzn. kostkami o 1-stajnym rozkadzie prawdopodobiestwa na 
   1 .. {@link #diceFaces}) i dodaj {@link #constantBonus}.
   Takie losowanie realizuje metoda {@link #random}. */
public class DiceRandom {

  /** Musi by >= 0. */
  public int dicesCount;
  /** Musi by > 0. */
  public int diceFaces;
  /** Moe by dowolne, take ujemne. */
  public int constantBonus;

  /** Obvious constructor, creates object with given attributes. */
  public DiceRandom(int aDicesCount, int aDiceFaces, int aConstantBonus)
  {
    dicesCount    = aDicesCount;
    diceFaces     = aDiceFaces;
    constantBonus = aConstantBonus;
  }
  
  public DiceRandom(int aDicesCount, int aDiceFaces)
  {
    this(aDicesCount, aDiceFaces, 0);
  }
  
  public DiceRandom(int aConstantBonus)
  {
    this(0, 0, aConstantBonus);
  }

  public int random ()
  {
     int res = constantBonus;
     int i;

     for (i = 1; i <= dicesCount; i++)
         res += Utils.rand.nextInt (diceFaces) + 1;
     return res;
  }

  /** Zwraca string AkB+C gdzie A = {@link #dicesCount},
     B = {@link #diceFaces}, C = {@link #constantBonus}. 
     "k" to skrt od "kostka", po angielsku pisze si "d" jak "dice".
     Wyjtki: jeli A = 0 zwraca C, jeli C = 0 (i A <> 0) zwraca AkB.  */
  public String toString ()
  {
    if (dicesCount == 0)
      return "" + constantBonus; else
      return dicesCount +"k" +diceFaces 
        +(constantBonus == 0 ? "" : "+" +constantBonus);
  }
}

/* 
 * $Log: DiceRandom.java,v $
 * Revision 1.1  2004/02/18 14:37:28  michalis
 * * pl stringi
 * * --help output changed (to reflect that those classes will be run by bin/scripts)
 * preparations for ClientApplet:
 * * Client rozdzielony na Client i ClientStandalone
 * * znaczna cz pakietu common przeniesiona do nowego pakietu game
 * X* make jamy_i_nory_client_applet.jar
 *
 * Revision 1.5  2004/02/09 23:08:50  michalis
 * + GPL license clause i copyrighty (moje i gdzieniegdzie Michaa, sorry Rafa
 *   ale naprawd nic nie zostao) do wszystkich *.java
 *
 * Revision 1.4  2004/01/18 20:56:27  michalis
 * * rand moved from DiceRandom to Utils (as final static)
 *
 * Revision 1.3  2004/01/18 18:44:57  michalis
 * + DiceRandom(int), Race.unarmedDamage, Race.skinDefense, Being.armorDefense, Being.weaponHit
 *
 * Revision 1.2  2004/01/18 17:49:59  michalis
 * * DiceRandom.toString slightly improved (+ tests for it in BaseTests),
 * + Race.toDetailString (* used in JamySingle)
 *
 * Revision 1.1  2004/01/18 06:26:33  michalis
 * * HitModifier ranamed to DiceRandom (more general name, this class may
 *   be used in many other cases when we need the AkB+C kind of randomness),
 *   small changes: + constantBonus, - parameterless constructor (useless).
 *
 *
 * Old CVS logs when this file was named "HitModifier.java":
 * Revision 1.7  2004/01/15 13:19:42  emes
 * * s/Pullic/Public/
 *
 * Revision 1.6  2004/01/10 03:08:58  michalis
 * * Many small fixes; mainly improvements to javadoc comments
 *
 * Revision 1.5  2004/01/08 23:48:19  michalis
 * + Minor changes and prepared for javadoc; also, commited to test mail notification
 *
 * Revision 1.4  2004/01/08 22:45:38  michalis
 * - Usunalem wlasny blad - te "reczne" logi ktore zaczalem tworzyc
 *
 * Revision 1.3  2004/01/08 22:37:42  michalis
 * + CVS Logs appended to all .java files
 *
 *
 */
