/*
  Copyright 2004 Michalis Kamburelis.

  This file is part of "Jamy & Nory".

  "Jamy & Nory" is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  "Jamy & Nory" is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with "Jamy & Nory"; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package jamy_i_nory.game;

/** Ta klasa przechowuje stan gracza nieaktywnego, tzn. gracza ktry
   nie jest martwy ale chwilowo nie uczestniczy w grze. Idea jest taka
   e ta klasa powinna by zupenie oderwana od klasy World i tym
   podobnych (jakby "bardziej globalnych") rzeczy (w przeciwiestwie
   do klasy Player ktra wanie stara si by moliwie zintegrowana ze
   wiatem w ktrym przebywa gracz). Aby t ide zrealizowa w bezpieczny
   i elegancki sposb zdecydowaem si nie czyni klasy Player
   podklas InactivePlayer (ani na odwrt) ani nie dodawa do klasy Player 
   pola w rodzaju isActive. Klasa InactivePlayer jest po prostu czym
   odrbnym. 
   
   <p>Zwracam uwag e ta klasa jest niemodyfikowalna (immutable).
   I taka ma pozosta, to usprawiedliwia uywanie tak wiele razy final
   zamiast tworzy metody "get".
   Ponadto nie moesz modyfikowa agregowanych w tej klasie obiektw,
   np. nie ma jzykowego zabezpieczenia przed dodawaniem przedmiotw 
   do {@link #backpack} w tej klasie ale przyjmijmy e nie moesz tego 
   robi. 
   
   <p>Wyjtkiem jest referencja do nowLocation ktr trzymamy w tej klasie
   a ktra odwouje si do elementw globalnych wiata i ktrej stan 
   bez wtpienia moe si zmienia podczas gdy dany gracz jest nieaktywny
   (np. inne postacie mog w tym czasie wchodzi i wychodzi z tej lokacji). 
   Nie stwarza to jednak problemw technicznych bo zbir lokacji w wiecie
   gry jest stay (patrz komentarze przy {@link World#locations}).
*/
public class InactivePlayer {

  final public Race race;
  final public StatModifier raceRandomStat;
  final public String name;
  final ItemsBag backpack;
  
  private StatModifyingItem[] fUsedItems = new StatModifyingItem[Being.USED_COUNT];
  
  public StatModifyingItem usedItem(int num)
  {
    return fUsedItems[num];
  }
  
  final public int life;
  
  final public Location nowLocation;
  
  /** Zwraca podstawowe informacje o nieaktywnym graczu. Intencj jest
     aby wynik moliwie przypomina Player.toString (ale tylko moliwie,
     bo np. InactivePlayer nie ma id) */     
  public String toString()
  {
    return "\"" +name +"\" " +race +" {ycie: " +life +"}";
  } 
  
  /** Na podstawie player tworzy obiekt InactivePlayer czyli czyni gracza
     nieaktywnym. Mona to nazwa "zamraaniem" obiektu Player. 
     Pod koniec tego konstruktora wywoujemy player.goInactive(),
     z czego wynika m.in. e po skonstruowaniu obiektu InactivePlayer 
     w ten sposb musisz zaprzesta odwoywania si gdziekolwiek w programie 
     do przekazanego tu obiektu player. */
  public InactivePlayer(Player player)
  {
    race = player.race;
    raceRandomStat = player.raceRandomStat;
    name = player.name;
    backpack = player.backpack;
    for (int i = 0; i < Being.USED_COUNT; i++)
      fUsedItems[i] = player.usedItem(i);
    life = player.life;
    nowLocation = player.nowLocation();
    
    player.goInactive();
  }
}

/*
 * $Log: InactivePlayer.java,v $
 * Revision 1.1  2004/02/18 14:37:28  michalis
 * * pl stringi
 * * --help output changed (to reflect that those classes will be run by bin/scripts)
 * preparations for ClientApplet:
 * * Client rozdzielony na Client i ClientStandalone
 * * znaczna cz pakietu common przeniesiona do nowego pakietu game
 * X* make jamy_i_nory_client_applet.jar
 *
 * Revision 1.2  2004/02/09 23:08:50  michalis
 * + GPL license clause i copyrighty (moje i gdzieniegdzie Michaa, sorry Rafa
 *   ale naprawd nic nie zostao) do wszystkich *.java
 *
 * Revision 1.1  2004/02/06 04:44:40  michalis
 * + InactivePlayer
 * + dodatkowy konstruktor do Being
 * + dodatkowy konstruktor Player(World, InactivePlayer)
 * * beforeDeath rozszerzone o parametr goingInactive
 * + Being.goInactive
 * * listToString moved from World to Utils
 * + ServerCommand : polecenie "mrozonki"
 *
 */

