/*
  Copyright 2004 Michalis Kamburelis.

  This file is part of "Jamy & Nory".

  "Jamy & Nory" is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  "Jamy & Nory" is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with "Jamy & Nory"; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package jamy_i_nory.game;

import java.util.*;

/** Lista obiektw InactivePlayer, no nulls, rnych nie tylko w sensie
   referencji ale wrcz o rnym name.
   
   <p>TODO: wymuszanie eby name byy rne jeszcze nie jest tu zrobione; 
   eleganckie wykonanie tego wymaga eby obiekty world.players() te miay 
   rne name i eby suma world.players() + ta lista te miaa tylko rne 
   names. */
public class InactivePlayersList extends LinkedList {
  
  /** Szuka obiektu o podanym name (name musi byc != null).
     Zwraca null jeli nie znalaz. */  
  public InactivePlayer findByName(String name)
  {
    Iterator i = iterator();
    while (i.hasNext()) {
      InactivePlayer p = (InactivePlayer) i.next();
      if (p.name.equals(name)) return p;
    }
    return null;
  }
  
  /** Szuka obiektu o podanym name (name musi byc != null) i (o ile znajdzie)     
     usuwa z listy. Zwraca null jeli nie znalaz. */  
  public InactivePlayer findAndRemoveByName(String name)
  {
    Iterator i = iterator();
    while (i.hasNext()) {
      InactivePlayer p = (InactivePlayer) i.next();      
      if (p.name.equals(name)) { i.remove(); return p; }
    }
    return null;
  }
}

/*
 * $Log: InactivePlayersList.java,v $
 * Revision 1.1  2004/02/18 14:37:28  michalis
 * * pl stringi
 * * --help output changed (to reflect that those classes will be run by bin/scripts)
 * preparations for ClientApplet:
 * * Client rozdzielony na Client i ClientStandalone
 * * znaczna cz pakietu common przeniesiona do nowego pakietu game
 * X* make jamy_i_nory_client_applet.jar
 *
 * Revision 1.2  2004/02/09 23:08:50  michalis
 * + GPL license clause i copyrighty (moje i gdzieniegdzie Michaa, sorry Rafa
 *   ale naprawd nic nie zostao) do wszystkich *.java
 *
 * Revision 1.1  2004/02/06 06:21:01  michalis
 * + PlayerConnection: dodatkowy konstruktor
 * + InactivePlayersList
 * * rozszerzona funkcjonalnoc Connection (musi umie wywoa oba konstruktory
 *   PlayerConnection)
 *   Dziaa ! Zamraanie i odmraanie graczy zrobione !
 *
 */
