/*
  Copyright 2004 Michalis Kamburelis.

  This file is part of "Jamy & Nory".

  "Jamy & Nory" is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  "Jamy & Nory" is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with "Jamy & Nory"; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package jamy_i_nory.game;

import java.util.*;

/** Lista zawierajca obiekty klasy {@link LinkToLocation}.

   <p>Nie ma w tym momencie na to adnego zabezpieczenia (TODO?), 
   ale ta lista moe zawiera tylko rne (w sensie referencji)
   elementy klasy {@link LinkToLocation LinkToLocation}. adnych nulli.
   
   <p>It is not guaranteed that this class with always inherit from
   LinkedList but it is guaranteed that it will implement List
   interface (with all optional features). (I wanted to avoid
   creating additional interface for this) */
public class LinkToLocationList extends IdedList {

  /** Find item that has {@link LinkToLocation#shortName shortName}
     equal to aShortName. Return null if such item does not exist.
     
     <p>This method does not inspect does aShortName prefix matches
     some prefix of some item on this list. Such functionality 
     should not be implemented here. */
  public LinkToLocation findShortName(String aShortName) {
    Iterator i = iterator();
    while (i.hasNext())
    {
      LinkToLocation l = (LinkToLocation) i.next();
      if (l.shortName.equals(aShortName)) return l;
    }
    return null;
  }
  
} 

/*
 * $Log: LinkToLocationList.java,v $
 * Revision 1.1  2004/02/18 14:37:28  michalis
 * * pl stringi
 * * --help output changed (to reflect that those classes will be run by bin/scripts)
 * preparations for ClientApplet:
 * * Client rozdzielony na Client i ClientStandalone
 * * znaczna cz pakietu common przeniesiona do nowego pakietu game
 * X* make jamy_i_nory_client_applet.jar
 *
 * Revision 1.4  2004/02/09 23:08:50  michalis
 * + GPL license clause i copyrighty (moje i gdzieniegdzie Michaa, sorry Rafa
 *   ale naprawd nic nie zostao) do wszystkich *.java
 *
 * Revision 1.3  2004/01/18 22:49:45  michalis
 * * LinkToLocationList now descends from IdedList
 *
 * Revision 1.2  2004/01/13 05:15:41  michalis
 * * docs improved
 *
 * Revision 1.1  2004/01/13 04:38:55  michalis
 * + JamySingle (single-user version of game), LinkToLocationList, UserCommand
 *
 */
