/*
  Copyright 2004 Michalis Kamburelis.

  This file is part of "Jamy & Nory".

  "Jamy & Nory" is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  "Jamy & Nory" is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with "Jamy & Nory"; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package jamy_i_nory.game;

import java.util.*;
import jamy_i_nory.common.*;

/** Lokacja w wiecie gry. */ 
public abstract class Location {

  /** Dugi (powiedzmy: kulku[nasto]linijkowy) opis lokacji.
     To powinien by zestaw kilku[nastu] penych zda, innymi
     sowy zaczynajcie z duej litery i koczcie zdania kropk.
     Zawsze przed wypisaniem description wyswietlimy take
     {@link #name name} wic description nie musi ju zawiera tego co zostao
     powiedziane w {@link #name name}. Inaczej ni {@link #name name}, 
     description moe zawiera zdania mwice do usera per ty, 
     np. dla {@link #name name} = "Tyek dinozaura" description moe
     brzmie "Stoisz na rodku tyka dinozaura."
     
     <p>Nie ma by amany na wiersze okrelonej dugoci 
     - sami je bdziemy ama przy 
     wywietlaniu na ekran uytkownika. Natomiast moe zawiera
     znaki nowej linii (brane z {@link Utils#nl Utils.nl}, nie uywajcie
     "\n", {@link Utils#nl Utils.nl} jest cross-platform) jeli
     chcecie wymusi zamanie linii w jakim miejscu. Dopuszczamy
     nawet uycie dwch {@link Utils#nl Utils.nl} pod rzd aby 
     zaznaczy co jak nowy akapit w tekcie. 
     Na kocu description nie ma by adnych znakw nowej linii. */
  public abstract String description();
  
  /** Krtka nazwa lokacji. To ma by nawyej kilka sw, bez adnego
     znaku nowej linii. Nie ma by zakoczone kropk, chocia
     powinno dobrze brzmie jako rwnowanik zdania (innymi sowy,
     pewnie najczciej bdziemy to wypisywali dopisujc kropk
     na kocu; ale nie zawsze). Ma si zaczyna z duej litery. 
     
     <p>Dla dobra usera powinnimy si stara eby wszystkie lokacje 
     miay moliwie unikalne nazwy. */
  final public String name;
  
  private StatModifier fStatModifier = new StatModifier();  
  
  /** Modyfikator aplikowany wszystkim istotom (Being) w tej lokacji.
     Nigdy null. */
  public StatModifier statModifier() { return fStatModifier; }
  
  private IdedList fCreatures = new IdedList();
  
  /** Lista Creatures obecnych w danej lokacji. 
  
     <p>Nie okrelamy czy implementacj jest ArrayList czy LinkedList 
     (czy ew. jeszcze co innego) wic tradycyjnie starajcie si pisa
     rzeczy w sposb ktry bdzie dziaa zawsze moliwie szybko
     (np. uywajcie iteratorw zamiast indeksowa list przy
     uyciu ptli for i metody get).
     
     <p>Nie okrelamy te czy implementacja tej funkcji to bdzie
     proste zwrcenie jakiego wewntrznego pola czy te bardziej
     skomplikowany proces bdzie potrzebny do wyliczenia wartoci
     tej funkcji. Wic nie wywouj tej funkcji co chwila jeli
     nie musisz, lepiej przechowaj sobie jej wynik (ale uwaga:
     podczas przechowywania tego wyniku pamitaj nie uywa 
     innych funkcji ktre mog modyfikowa list creatures w danej
     komnacie, jak {@link Creature#setNowLocation 
     Creature.setNowLocation}).
     
     <p>Nie ma w tym momencie na to adnego zabezpieczenia (TODO?), 
     ale ta lista moe zawiera tylko rne (w sensie referencji)
     elementy klasy {@link Creature Creature}. adnych nulli. 
     
     <p>Mona modyfikowa bezporednio tyko z {@link Creature#setNowLocation 
     Creature.setNowLocation}. Notki o aktualnej implementacji:
     przechowujemy tutaj cay czas list aktualnych creatures w danej
     komnacie. Std potrzeba uaktualniania jej w setNowLocation
     (wywoywanej take wszdzie tam gdzie uaktualniamy world.creatures()).
     To zapewnia nam szybko dziaania, nawet dla sytuacji gdy bdziemy 
     mieli w grze bardzo duo creatures(). */
  public IdedList creatures() { return fCreatures; }
  
  private IdedList fPlayers = new IdedList();
  
  /** Lista graczy w danej lokacji. 
          
     <p>Uwagi podobne jak przy  {@link #creatures creatures()}: 
     nie definiujemy implementacji interfejsu List i
     nie definiujemy czy ta funkcja dziaa szybko czy te robi 
     jakie obliczenia by uzyska wynik. Pamitaj nie przechowywa
     wartoci zwrconej przez t funkcj jeli robisz rzeczy ktre
     mog sprawi e lista zwracana przez t funkcj bdzie inna.

     <p>Ta lista moe zawiera tylko rne (w sensie referencji)
     elementy klasy {@link Player Player}. adnych nulli. 
     
     <p>Mona modyfikowa bezporednio tyko z {@link Player#setNowLocation 
     Player.setNowLocation}. Notki o aktualnej implementacji
     takie jak przy {@link #creatures creatures()}. */
  public IdedList players() { return fPlayers; }
  
  /** Lista Beings w danej lokacji, czyli suma creatures() i players().
     Nie mona jej modyfikowa.
     
     <p>Uwagi podobne jak przy  {@link #creatures creatures()}: 
     nie definiujemy implementacji interfejsu List i
     nie definiujemy czy ta funkcja dziaa szybko czy te robi 
     jakie obliczenia by uzyska wynik. Pamitaj nie przechowywa
     wartoci zwrconej przez t funkcj jeli robisz rzeczy ktre
     mog sprawi e lista zwracana przez t funkcj bdzie inna.

     <p>Ta lista moe zawiera tylko rne (w sensie referencji)
     elementy klasy {@link Being Being}. adnych nulli. */
  public IdedList beings()
  {
    IdedList result = new IdedList(creatures());
    result.addAll(players());
    return result;
  }
  
  private IdedList fItems = new IdedList();
  
  /** Lista Items w danej lokacji. 
      
     <p>Uwagi podobne jak przy  {@link #creatures creatures()}: 
     nie definiujemy implementacji interfejsu List i
     nie definiujemy czy ta funkcja dziaa szybko czy te robi 
     jakie obliczenia by uzyska wynik.

     <p>Nie ma w tym momencie na to adnego zabezpieczenia (TODO?), 
     ale ta lista moe zawiera tylko rne (w sensie referencji)
     elementy klasy {@link Item Item}. adnych nulli.

     <p>W tym momencie ta lista jest swobodnie modyfikowalna
     skdkolwiek (tzn. dopki spenione s warunki podane wyej). */
  public IdedList items() { return fItems; }
  
  private LinkToLocationList fLinks = new LinkToLocationList();
  
  /** Lista przej z danej lokacji. Moe by modyfikowana
     i zmieniana w czasie gry ale NIE bezporednio, tylko przez
     metody {@link #addLink}, {@link #removeLink}. 
     W ten sposb mona zrobi rzeczy 
     tak banalne jak otwieranie drzwi ale te tak ciekawe
     jak np. jaki gracz wykona gdzie jak akcj i w rezultacie
     w zupenie innej lokacji otwiera si ukryte dotychczas przejcie.
     
     <p>Uwagi podobne jak przy  {@link #creatures creatures()}: 
     nie definiujemy implementacji interfejsu List i
     nie definiujemy czy ta funkcja dziaa szybko czy te robi 
     jakie obliczenia by uzyska wynik. */
  public LinkToLocationList links() { return fLinks; }
  
  /** Dodaj link to listy {@link #links}. Podany link nie moe ju wystpowa
     na tej licie. Wywietla odpowiednie komunikaty przez putMessages graczom
     w tej komnacie. */
  public void addLink(LinkToLocation link)
  {
    links().add(link);
    tellEveryone("W tym miejscu pojawia si nowe przejcie [" +link.shortName +"].");
  }
  
  /** Usu link z listy {@link #links}. Jeeli podany link nie wystpuje na licie
     to dziaa jak NOP (i zwraca false). Wpp. usuwa podany link,
     powiadamiajc wszystkich w tej komnacie przez putMessage, i zwraca
     true. Czyli zawsze zwraca "czy link by na licie links". */
  public boolean removeLink(LinkToLocation link)
  {
    boolean result = links().remove(link);
    if (result)    
      tellEveryone("Przejcie [" +link.shortName +"] przestaje by dostpne.");
    return result;
  }
  
  /** Utility routine: create two LinkToLocation objects.
  
     <ul>
       <li>First one with shortName = aShortNameToTarget, 
         longName = aLongNameToTarget, target = target. 
         Add this link to this.links().
       <li>Second one with shortName = aShortNameToSource, 
         longName = aLongNameToSource, target = this. 
         Add this link to target.links().
     </ul>
     
     and connect those links with their wayBack fields. */
  public void createTwoSidedLinks(Location target,
    String aShortNameToTarget, String aLongNameToTarget,
    String aShortNameToSource, String aLongNameToSource)
  {
    LinkToLocation toTarget = new LinkToLocation(target, aShortNameToTarget, 
      aLongNameToTarget);      
    LinkToLocation toSource = new LinkToLocation(this, aShortNameToSource, 
      aLongNameToSource);
    this  .links().add(toTarget);
    target.links().add(toSource);
    
    toTarget.setWayBack(toSource);
    toSource.setWayBack(toTarget);
  }
  
  /** wiat w ktrym jest dana lokacja. Cannot be null. 
     world.location musi zawiera ten obiekt. */
  public World world;
  
  /** Obvious constructor. Create Location with given properties. */
  public Location(World aWorld, String aName, StatModifier aStatModifier)
  {
    world = aWorld;
    name = aName;  
    fStatModifier = aStatModifier;
  }
  
  /** Obvious constructor. Create Location with given properties,
    statModifier will be set to zero-modifier (that doesn't modify
    any stats).. */
  public Location(World aWorld, String aName)
  {
    this(aWorld, aName, new StatModifier());
  }

  /** Peny opis lokacji, opisuje wszystko co si znajduje w lokacji
     widziane z punktu widzenia {@link Being Being} observer
     w adnie sformatowanej formie. Zawiera znaki nowej linii,
     i zawsze koczy si znakiem nowej linii. 
     
     <p>Jeeli showObserverInLocation == false to gdy bdzie wypisywao
     beings() w danej lokacji to nie wypisze observera; zazwyczaj
     jest to podane (gracz zazwyczaj wie e sam jest w danej komnacie). */
  public String toString(Being observer, boolean showObserverInLocation)
  {
    /* TODO: na elementy items te powinnismy patrze z perspektywy observer,
       "czy observer zidentyfikowa dany przedmiot". 
       TODO: by moe przejcia te warto wypisywa z punktu widzenia
       observer, np. "czy widzi jakie przejcie / czy jest dla niego
       dostpne" */
    StringBuffer result = new StringBuffer(
      "----------------------------------------"  +Utils.nl+
      "*** " +name +" ***" +Utils.nl+ 
      description() +Utils.nl);
    
    { 
      /* Dopisz do result liste beings(). 
         Uzywaj b.toString(observer) (zamiast b.toString()).
         Nie dopisuj observer jesli not showObserverInLocation. */
      StringBuffer beingsDescr = new StringBuffer();
      Iterator i = beings().iterator();
      while (i.hasNext())
      {      
        Being b = (Being) i.next();
        if (showObserverInLocation || (b != observer)) 
          beingsDescr.append("  " +b.toString(observer) +Utils.nl);
      }
      if (beingsDescr.length() != 0)
        result.append("Znajduj si tu istoty:" +Utils.nl +beingsDescr);
    }
    
    result.append(Utils.collectionToMultiLineString(items(), 
        "Znajduj si tu przedmioty:", "  ")
      +Utils.collectionToMultiLineString(links(), 
         "Przejcia:", "  "));
         
    if (!statModifier().isZero())
      result.append("W tym miejscu wszystkie istoty maj zmodyfikowane cechy: " 
        +statModifier().toString() +Utils.nl);
         
    return result.toString();
  }
  
  /** Ta metoda zwraca co jak {@link #toString(Being,boolean) 
     toString(observer,true)} dla wszystkowiedzcego observera. 
     Uywa tylko do testowania. */
  public String toString() 
  {
    return name +"." +Utils.nl
      +description() +Utils.nl
      +Utils.collectionToMultiLineString(beings(), 
         "Znajduj si tu istoty:", "  ")
      +Utils.collectionToMultiLineString(items(), 
         "Znajduj si tu przedmioty:", "  ")
      +Utils.collectionToMultiLineString(links(), 
         "Przejcia:", "  ")
      +(statModifier().isZero() ? "" : 
         "W tym miejscu wszystkie istoty maj zmodyfikowane cechy: " 
         +statModifier().toString() +Utils.nl);
  }
  
  /** Stworzenie being uywa w lokacji przedmiotu item.     
     Uaktualnia putMessage wszystkich zainteresowanych, w szczeglnoci
     gwarantuje e bdzie wywoane przynajmniej raz being.putMessage
     (czyli being na pewno dostanie jak odpowied, chociaby "Nie umiesz
     uy tego przedmiotu.").
     
     <p>W podklasach moesz to pokry aby zrobi rzeczy w rodzaju
     uycie klucza otwiera drzwi, uycie opaty otwiera przejcie przez
     jaskini, uycie magicznego talizmanu powoduje pojawienie si
     magicznego stwora w komnacie itd. Zawsze pokrywaj piszc co w rodzaju
     <pre>
     public String useItem(Being being, Item item)
     {
       if (item daje si uy w sposb specyficzny dla tej lokacji)
       {
         uyj item w tej lokacji
       } else
         super(being, item)
     }
     </pre>
     tzn. zawsze na kocu jako ostatni moliwo rb wywoanie nadklasy.
     
     <p>W klasie Location wywouje item.useItem() (czyli prbuje uy
     przedmiotu w sposb niezaleny od lokacji) - jeli si nie udao
     to mwi do being.putMessage("Przedmiotu co-tam nie da si w aden 
     sposb uy w tej lokacji.") i zwraca false.
     
     <p>Czasem nie jest takie jasne czy implementowa useItem w przedmiocie
     (tzn. pokrywajc useItem w podklasie Item) czy w lokacji
     (pokrywajc t metod w podklasie Location). Zwaszcza gdy akcja
     jest specyficzna i dla lokacji i dla rodzaju przedmiotu (jak np.
     KluczNoraBiblioteka w lokacjach Nora i Biblioteka). No c, mona w takim
     wypadku zaimplementowa metod gdzie si wydaje wygodniejsza/naturalniejsza.
     Podzia "kiedy implementowa Location.useItem a kiedy Item.useItem"
     nie jest precyzyjnie wyznaczony - po prostu moesz implementowa to 
     ktre bdzie wygodniej.
     
     @return Zwraca czy udao si w jaki sensowny sposb
     uy tego przedmiotu. Tzn. odpowied false oznacza e tego przedmiotu 
     nijak w aden sposb w ogle nie dao si w aden sensowny sposb
     zastosowa w tej lokacji. Np. uycie klucza A w lokacji z drzwiami
     ktre mona otworzy tylko kluczem B powinno da komunikat 
     bein.putMessage("Klucz A nie pasuje do drzwi B") i ta metoda powinna 
     zwrci true (bo dao si zrobi (prbowa zrobi) co sensownego 
     z kluczem A. Natomiast uycie klucza A w komnacie w ktrej w ogle 
     nie ma adnych drzwi ani nic innego powinno da 
     being.putMessage("Nie da si tego klucza w nic wetkn, kurka wodna.") 
     i ta metoda zwrci false. */
  public boolean useItem(Being being, Item item)
  {
    boolean result = item.useItem(being);
    if (!result)
      being.putMessage("Przedmiotu " +item 
        +" nie da si w aden sposb uy w tej lokacji.");
    return result;
  }
  
  /** Dla kadego gracza ktry jest w tej lokacji zrobi 
     player.putMessage(message). */
  public void tellEveryone(String message)
  {
    tellEveryone(message, null);
  }
  
  /** Jak {@link #tellEveryone(String)} tyle e
     tutaj NIE dodajemy wiadomoci dla gracza except, nawet jeli jest
     on w lokacji loc. Porwnujemy graczy przy pomocy referencji. 
     Moesz przekaza except == null albo except nie bdce klasy Player
     (ale np. klasy Creature), wtedy ta metoda zadziaa dokadnie
     tak jak {@link #tellEveryone(String)}. */
  public void tellEveryone(String message, Being except)
  {
    /* monaby zoptymalizowa dla (!except instanceof Player) i 
       (except == null) ale nie warto; lepiej mie elegancki kod ktry atwo 
       bdzie pniej zmienia. */
    Iterator i = world.players().iterator();
    while (i.hasNext())
    {
      Player p = (Player) i.next();
      if (p != except && p.nowLocation() == this)  
        p.putMessage(message);
    }
  }  

  /** Tak samo jak odpowiednia metoda {@link #tellEveryone(String,Being)} 
     tyle e tutaj zamiast podawa String podajesz obiekt
     speniajcy interfejs MessageGenerator ktry bdzie uywany aby
     wygenerowa string dla kadego gracza. */
  public void tellEveryone(MessageGenerator mesgGenerator, Being except)
  {
    /* monaby zoptymalizowa dla (!except instanceof Player) i 
       (except == null) ale nie warto; lepiej mie elegancki kod ktry atwo 
       bdzie pniej zmienia. */
    Iterator i = world.players().iterator();
    while (i.hasNext())
    {
      Player p = (Player) i.next();
      if (p != except && p.nowLocation() == this) 
        p.putMessage(mesgGenerator.genMessage(p));
    }  
  }
  
  /** Tak samo jak odpowiednia metoda {@link #tellEveryone(MessageGenerator,
     Being)} tyle e tutaj podajesz DWA obiekty dla ktrych NIE wypisywa
     wiadomoci. Wiem, wiem - z czasem by moe przerobi to na wersj
     z list albo tablic obiektw. Na razie dwa wystarcz. */
  public void tellEveryone(MessageGenerator mesgGenerator, Being except1,
    Being except2)
  {
    Iterator i = world.players().iterator();
    while (i.hasNext())
    {
      Player p = (Player) i.next();
      if (p.nowLocation() == this && p != except1 && p != except2) 
        p.putMessage(mesgGenerator.genMessage(p));
    }  
  }
}

/* 
 * $Log: Location.java,v $
 * Revision 1.1  2004/02/18 14:37:28  michalis
 * * pl stringi
 * * --help output changed (to reflect that those classes will be run by bin/scripts)
 * preparations for ClientApplet:
 * * Client rozdzielony na Client i ClientStandalone
 * * znaczna cz pakietu common przeniesiona do nowego pakietu game
 * X* make jamy_i_nory_client_applet.jar
 *
 * Revision 1.24  2004/02/15 15:24:35  michalis
 * + pdf generating
 * + COPYING file
 * + Utils.check
 * + Location.add/removeLink
 * + LocationWithDynLink
 * * NoraOrBiblioteka.useItem simplified
 * * teraz i w bibliotece i w norze s drzwi
 * - Makefiles rozsiane po katalogach.
 * * Globalny Makefile rzdzi.
 *
 * Revision 1.23  2004/02/14 11:01:51  michalis
 * * demo otwierania i zamykania drzwi kluczem zrobione
 *
 * Revision 1.22  2004/02/14 01:09:43  michalis
 * + przejcia w drug stron (LinkToLocation.wayBack, Location.createTwoSidedLinks)
 * * followLink -> Being
 * * lepsze komunkaty o przejciach w setNowLocation
 * * Troll osabiony
 *
 * Revision 1.21  2004/02/09 23:08:50  michalis
 * + GPL license clause i copyrighty (moje i gdzieniegdzie Michaa, sorry Rafa
 *   ale naprawd nic nie zostao) do wszystkich *.java
 *
 * Revision 1.20  2004/02/04 20:04:27  michalis
 * * wreszcie poprawione messages dla Being.attack
 * * Being.attack nie pozwala zaatakowa samego siebie
 * + UserCommand.parseId, uywane wszdzie w UserCommand.execute gdzie potrzebny id
 *
 * Revision 1.19  2004/02/04 14:00:24  michalis
 * * Being.tellEveryoneInNowLocation -> sayEveryoneInNowLocation
 *   (mniej mylca nazwa)
 * * Server powiadamia wszystkich graczy w komnacie e pojawi si nowy gracz
 * * PlayersMessages.tellEveryoneInLocation - docs updated,
 *   implemented in a cleaner way
 * + MessageGenerator
 * + MessageFormatter
 * * Being.attitude changed to realAttitude and observedAttitude
 * * Many fixes to use MessageFormatter when updating PlayersMessages.
 *   Many TO-DOs closed.
 * * Attitude.toStr translated to polish
 * * Poprawiony bug w Being.dead - Micha, nie mona tworzy PlayersMessages
 *   tu przed usuniciem gracza z gry - w czasie uywania PlayersMessages
 *   world.players() nie mog by modyfikowane. Z drugiej strony, to jest moja
 *   wina e jeszcze tego nie mona zrobi (bo to *powinno* by moliwe tylko
 *   musz zapisa inteligentniej PlayersMessages). OK, wanie mi przyszo do
 *   gowy jak to zrobi bardzo atwo i bardzo elegancko (klasa PlayersMessages
 *   w ogle nie bdzie potrzebna i unikniemy przerzucania wszdzie parametrw
 *   playersMessages) i w nastpnym commicie to bdzie zrobione (gupi, gupi,
 *   gupi jestem e od pocztku tego tak nie zrobiem).
 * * OK, serwer skunk.one.pl nie odpowiada wic nie bd commitowa, od razu
 *   zrobi to o czym mwiem powyej:
 *   + Player.getMessage, Player.putMessage
 *   + Location.tellEveryone (3 wersje)
 *   - PlayersMessages (funkcjonalno przerzucona w caoci do obiektw Player i
 *     Location)
 *   * masa uproszcze kodu (m.in. nie musimy ju przerzuca wszdzie
 *     parametrw PlayersMessages)
 *   - klasa Being.DeadAnswer (useless, teraz Being.dead() znowu zwraca prostego
 *     booleana)
 *   * w ogle Being.dead() jest teraz o niebo prostsze.
 *   Skoczyem! PlayersMessages usunite razem z wieloma niewygodami.
 *
 * Revision 1.18  2004/02/04 09:09:23  michalis
 * + contract of StatModifier.toString specified
 * + Location.statModifier(), odpowiednio rozszerzone Location.toString
 * * Being.fullStats odpowiednio rozszerzone
 * + lokacja Bagno (eby przetestowa Location.statModifier()),
 * * Goblin przeniesiony z tyka dinozura na zachodni balkon,
 *   Troll przeniesiony z tyka dinozura na wschodni balkon
 *   (eby ZPL nie pokazywa lokacji o nazwie "tyek dinozaura")
 *
 * Revision 1.17  2004/02/02 02:16:12  michalis
 * + PlayersMessages(Player,String) (unused for now)
 * + Item.useItem
 * + Item.ElfKillerWhistle implementujcy useItem
 * * Location.useItem wywouje Item.useItem
 * + Egzemplarz Item.ElfKillerWhistle dodany do salonu
 * * some docs clarified in Being
 *
 * Revision 1.16  2004/02/01 19:14:07  michalis
 * * poleceine "uzyj" renamed to "zaloz"
 * + Being.findItem
 * + Location.useItem
 * + polecenie "uzyj" zaimplementowane jako "uzyj cos w danej lokacji"
 * - some TO-DOs in UserCommand removed (done!)
 *
 * Revision 1.15  2004/01/19 02:25:56  michalis
 * * Location.creatures/players not calculated but updated in each setNowLocation
 *
 * Revision 1.14  2004/01/19 01:43:43  michalis
 * + world.creatures(), * Location.creatures() is now only calculated
 *
 * Revision 1.13  2004/01/15 19:47:34  emes
 * - removed obsolete code
 *
 * Revision 1.12  2004/01/15 19:47:00  emes
 * + added class IdedList for items with id
 * + method IdedList.byId() returns Ided object with specified id
 *
 * Revision 1.11  2004/01/15 14:34:02  emes
 * + objectById () method
 *
 * Revision 1.10  2004/01/13 04:38:55  michalis
 * + JamySingle (single-user version of game), LinkToLocationList, UserCommand
 *
 * Revision 1.9  2004/01/12 02:20:04  michalis
 * * docs improved
 *
 * Revision 1.8  2004/01/12 01:22:45  michalis
 * + Location/Being.toString(Being), czyli obiekty z punktu
 *   widzenia danego Being
 *
 * Revision 1.7  2004/01/12 00:33:15  michalis
 * + Location.players(), beings()
 *
 * Revision 1.6  2004/01/11 23:51:54  michalis
 * * Location.world added and necessary changes made, some Player docs improved
 *
 * Revision 1.5  2004/01/11 09:34:26  michalis
 * + Class LinkToLocation, Location.links and necessary tests and docs
 *
 * Revision 1.4  2004/01/11 07:40:21  michalis
 * * Poprawki i ulepszenia dokumentacji
 *
 * Revision 1.3  2004/01/11 06:51:55  michalis
 * + World class, Being.nowLocation
 *
 * Revision 1.2  2004/01/11 04:29:17  michalis
 * + Important comments to Location.creatures() added
 *
 * Revision 1.1  2004/01/11 04:15:35  michalis
 * + Class Location
 *
 */
