/*
  Copyright 2004 Michalis Kamburelis.

  This file is part of "Jamy & Nory".

  "Jamy & Nory" is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  "Jamy & Nory" is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with "Jamy & Nory"; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package jamy_i_nory.game;

import java.util.regex.*;
        
/** Celem tej klasy jest zaimplementowa interfejs 
   {@link MessageGenerator} w sposb ktry pozwoliby na atwiejsze
   uywanie go.
   
   <p>Uywajc tej klasy moesz zamiast
   <pre>
   MessageGenerator() {
     genMessage(Player p) {
       return "widzisz potwora " +c.toString(p)
     }
   }
   </pre>
   napisa 
   <pre>
   new MessageFormatter("widzisz potwora %1", c)
   </pre> */
public class MessageFormatter implements MessageGenerator { 
  private String formatString;
  private Being being1, being2;
  private Pattern proc1Pattern = Pattern.compile("%1"),
                  proc2Pattern = Pattern.compile("%2");
  
  /** Tworzy obiekt ktry za kadym razem w genMessage(p) bdzie zwraca
     aFormatString zamieniajc w nim wszystkie wystpienia "%1" na
     aBeing1.toString(p) (czyli na opis aBeing1 z punktu widzenia gracza p). 
     TODO: W przyszoci rozszerzymy moliwoci aFormatString aby mg
     zamienia inne stringi w rodzaju %liczba. TODO: Chwilowo nie uywaj znaku 
     "%" w aFormatString,  w przyszoci trzeba bdzie podwaja znak "%". */
  public MessageFormatter(String aFormatString, Being aBeing1)
  {
    this(aFormatString, aBeing1, null);
  }
  
  /** Tworzy obiekt ktry za kadym razem w genMessage(p) bdzie zwraca
     aFormatString zamieniajc w nim wszystkie wystpienia
     "%1" na aBeing1.toString(p), "%2" na aBeing2.toString(p). */
  public MessageFormatter(String aFormatString, Being aBeing1, Being aBeing2)
  {
    formatString = aFormatString;
    being1 = aBeing1;
    being2 = aBeing2;
  }
  
  public String genMessage(Player p)
  {
    String result = proc1Pattern.matcher(formatString).replaceAll(being1.toString(p));
    if (being2 != null)
      result = proc2Pattern.matcher(result).replaceAll(being2.toString(p));
    return result;
  }
}

/*
 * $Log: MessageFormatter.java,v $
 * Revision 1.1  2004/02/18 14:37:28  michalis
 * * pl stringi
 * * --help output changed (to reflect that those classes will be run by bin/scripts)
 * preparations for ClientApplet:
 * * Client rozdzielony na Client i ClientStandalone
 * * znaczna cz pakietu common przeniesiona do nowego pakietu game
 * X* make jamy_i_nory_client_applet.jar
 *
 * Revision 1.2  2004/02/09 23:08:50  michalis
 * + GPL license clause i copyrighty (moje i gdzieniegdzie Michaa, sorry Rafa
 *   ale naprawd nic nie zostao) do wszystkich *.java
 *
 * Revision 1.1  2004/02/04 14:00:25  michalis
 * * Being.tellEveryoneInNowLocation -> sayEveryoneInNowLocation
 *   (mniej mylca nazwa)
 * * Server powiadamia wszystkich graczy w komnacie e pojawi si nowy gracz
 * * PlayersMessages.tellEveryoneInLocation - docs updated,
 *   implemented in a cleaner way
 * + MessageGenerator
 * + MessageFormatter
 * * Being.attitude changed to realAttitude and observedAttitude
 * * Many fixes to use MessageFormatter when updating PlayersMessages.
 *   Many TO-DOs closed.
 * * Attitude.toStr translated to polish
 * * Poprawiony bug w Being.dead - Micha, nie mona tworzy PlayersMessages
 *   tu przed usuniciem gracza z gry - w czasie uywania PlayersMessages
 *   world.players() nie mog by modyfikowane. Z drugiej strony, to jest moja
 *   wina e jeszcze tego nie mona zrobi (bo to *powinno* by moliwe tylko
 *   musz zapisa inteligentniej PlayersMessages). OK, wanie mi przyszo do
 *   gowy jak to zrobi bardzo atwo i bardzo elegancko (klasa PlayersMessages
 *   w ogle nie bdzie potrzebna i unikniemy przerzucania wszdzie parametrw
 *   playersMessages) i w nastpnym commicie to bdzie zrobione (gupi, gupi,
 *   gupi jestem e od pocztku tego tak nie zrobiem).
 * * OK, serwer skunk.one.pl nie odpowiada wic nie bd commitowa, od razu
 *   zrobi to o czym mwiem powyej:
 *   + Player.getMessage, Player.putMessage
 *   + Location.tellEveryone (3 wersje)
 *   - PlayersMessages (funkcjonalno przerzucona w caoci do obiektw Player i
 *     Location)
 *   * masa uproszcze kodu (m.in. nie musimy ju przerzuca wszdzie
 *     parametrw PlayersMessages)
 *   - klasa Being.DeadAnswer (useless, teraz Being.dead() znowu zwraca prostego
 *     booleana)
 *   * w ogle Being.dead() jest teraz o niebo prostsze.
 *   Skoczyem! PlayersMessages usunite razem z wieloma niewygodami.
 *
 */
