/*
  Copyright 2004 Michalis Kamburelis.

  This file is part of "Jamy & Nory".

  "Jamy & Nory" is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  "Jamy & Nory" is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with "Jamy & Nory"; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package jamy_i_nory.game;

/** Prosty interfejs specyfikujcy metod ktra dla danego gracza zwraca
   String. Intencj jest aby klasy speniajce ten interfejs byy
   podawane jako parametry dla metod w rodzaju {@link Location#tellEveryone}  
   i uywane byy aby generowa w rodku stringi "z punktu widzenia
   gracza p" - np. zamiast przekazywa w lokacji string
   "widzisz potwora " +c.toString() lepiej zrobi
   <pre>
   MessageGenerator() {
     genMessage(Player p) {
       return "widzisz potwora " +c.toString(p)
     }
   }
   </pre>
   W ten sposb potwr c jest kadorazowo opisywany <b>tak jak go widzi
   odpowiedni gracz</b> - np. dla jednego gracza c bdzie widoczny jako
   hostile, dla innego (o mniejszej inteligencji) bdzie unknown. 
   
   <p>Po wygodn klas, ktra pozwala skrci poniszy przykad do jednej
   linijki, patrz {@link MessageFormatter}. */
public interface MessageGenerator {
  public String genMessage(Player p);
}

/*
 * $Log: MessageGenerator.java,v $
 * Revision 1.1  2004/02/18 14:37:30  michalis
 * * pl stringi
 * * --help output changed (to reflect that those classes will be run by bin/scripts)
 * preparations for ClientApplet:
 * * Client rozdzielony na Client i ClientStandalone
 * * znaczna cz pakietu common przeniesiona do nowego pakietu game
 * X* make jamy_i_nory_client_applet.jar
 *
 * Revision 1.2  2004/02/09 23:08:50  michalis
 * + GPL license clause i copyrighty (moje i gdzieniegdzie Michaa, sorry Rafa
 *   ale naprawd nic nie zostao) do wszystkich *.java
 *
 * Revision 1.1  2004/02/04 14:00:25  michalis
 * * Being.tellEveryoneInNowLocation -> sayEveryoneInNowLocation
 *   (mniej mylca nazwa)
 * * Server powiadamia wszystkich graczy w komnacie e pojawi si nowy gracz
 * * PlayersMessages.tellEveryoneInLocation - docs updated,
 *   implemented in a cleaner way
 * + MessageGenerator
 * + MessageFormatter
 * * Being.attitude changed to realAttitude and observedAttitude
 * * Many fixes to use MessageFormatter when updating PlayersMessages.
 *   Many TO-DOs closed.
 * * Attitude.toStr translated to polish
 * * Poprawiony bug w Being.dead - Micha, nie mona tworzy PlayersMessages
 *   tu przed usuniciem gracza z gry - w czasie uywania PlayersMessages
 *   world.players() nie mog by modyfikowane. Z drugiej strony, to jest moja
 *   wina e jeszcze tego nie mona zrobi (bo to *powinno* by moliwe tylko
 *   musz zapisa inteligentniej PlayersMessages). OK, wanie mi przyszo do
 *   gowy jak to zrobi bardzo atwo i bardzo elegancko (klasa PlayersMessages
 *   w ogle nie bdzie potrzebna i unikniemy przerzucania wszdzie parametrw
 *   playersMessages) i w nastpnym commicie to bdzie zrobione (gupi, gupi,
 *   gupi jestem e od pocztku tego tak nie zrobiem).
 * * OK, serwer skunk.one.pl nie odpowiada wic nie bd commitowa, od razu
 *   zrobi to o czym mwiem powyej:
 *   + Player.getMessage, Player.putMessage
 *   + Location.tellEveryone (3 wersje)
 *   - PlayersMessages (funkcjonalno przerzucona w caoci do obiektw Player i
 *     Location)
 *   * masa uproszcze kodu (m.in. nie musimy ju przerzuca wszdzie
 *     parametrw PlayersMessages)
 *   - klasa Being.DeadAnswer (useless, teraz Being.dead() znowu zwraca prostego
 *     booleana)
 *   * w ogle Being.dead() jest teraz o niebo prostsze.
 *   Skoczyem! PlayersMessages usunite razem z wieloma niewygodami.
 *
 */
