/*
  Copyright 2004 Michalis Kamburelis.

  This file is part of "Jamy & Nory".

  "Jamy & Nory" is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  "Jamy & Nory" is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with "Jamy & Nory"; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
 
package jamy_i_nory.game;

/* This class implements realAttitude using boolean attributes 
  {@link #attackPlayers} and {@link #attackCreatures}. */
public class SimpleMindedCreature extends Creature {

  /** Is this creature hostile to {@linkplain Player players} ? */
  public boolean attackPlayers;
  /** Is this creature hostile to other creatures ? */
  public boolean attackCreatures;
  
  public SimpleMindedCreature (World aWorld, Location startLocation, 
    Race aRace, String aName, int aWanderingFactor, 
    boolean aAttackPlayers, boolean aAttackCreatures)
  {
    super(aWorld, startLocation, aRace, aName, aWanderingFactor);
    attackPlayers = aAttackPlayers;
    attackCreatures = aAttackCreatures;
  }
  
  /** Returns HOSTILE for Player if and only if attackPlayers.<br>
      Returns HOSTILE for Creature if and only if attackCreatures. */
  public int realAttitude (Being target)
  {    
    if (target instanceof Player) 
      return Attitude.isHostileToAttitude(attackPlayers); else
      return Attitude.isHostileToAttitude(attackCreatures);
  }
}

/*
 * $Log: SimpleMindedCreature.java,v $
 * Revision 1.1  2004/02/19 15:32:40  michalis
 * * Creature.attackPlayers/Creatures -> SimpleMindedCreature
 * + NonHostileCreature
 * + ElfFairy, ElfFairyWhistle
 * + wanderingFactor, teraz prawie wszystkie creatures wasaj si po wiecie
 *
 */
