/*
  Copyright 2004 Michalis Kamburelis, Micha Saaban.

  This file is part of "Jamy & Nory".

  "Jamy & Nory" is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  "Jamy & Nory" is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with "Jamy & Nory"; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package jamy_i_nory.game;

/** Stats modifier, used for races, items and locations. */
public class StatModifier {

  /* Modifiers that are summed. 
     To add new additible modifier it is sufficient to 
     - increase MODIFS_COUNT
     - add new item to addModifsNames
     You can also
     - add new param to StatModifier(int, int, ...) constructor
     - add new method public int xxx() { return addModifs[xxx]; }
     - add new const MODIF_xxx
  */

  public final static int MODIF_STRENGTH     = 0;
  public final static int MODIF_DEXTERITY    = 1;
  public final static int MODIF_MAGIC        = 2;
  public final static int MODIF_MAX_LIFE     = 3;
  public final static int MODIF_INTELLIGENCE = 4;
  public final static int MODIFS_COUNT       = 5;
  public final static String addModifsNames[] = 
    { "sia", "zrczno", "magia", "max ycie", "inteligencja" };  

  private int addModifs[] = new int[MODIFS_COUNT];
  
  public StatModifier (int strength, int dexterity, int magic, 
    int maxLife, int intelligence)
  {
    addModifs[MODIF_STRENGTH    ] = strength;
    addModifs[MODIF_DEXTERITY   ] = dexterity;
    addModifs[MODIF_MAGIC       ] = magic;
    addModifs[MODIF_MAX_LIFE    ] = maxLife;
    addModifs[MODIF_INTELLIGENCE] = intelligence;
  }

  /**Taki konstruktor jest bardziej elastyczny niz poprzedni - nie bedziemy
    *musieli go modyfikowac w razie ewentualnego dodawania/usuwania modyfikatorow. 
    *Sensowno jego istnienia zalezy od ilosci dodatkowych przyszych modyfikatorw;
    *Patrz rwnie @link #getModifier() getModifier.*/
  
  public StatModifier (int[] stat) {
    if (stat != null) {
      if (stat.length >= MODIFS_COUNT)
          for (int i = 0; i < MODIFS_COUNT; i++) 
            addModifs[i] = stat[i];
    }
  }

  /** Create a StatModifier that doesn't modify any stats. */
  public StatModifier () { ; }

  /**Jedna, zgrabna metoda 'get' dla wszystkich modyfikatorow. <rafa>
    *  @param i modyfikator ktorego wartosc chcemy otrzymac
    *  @return wartosc modyfikatora i
    */
  public int getModifier (int i) { return addModifs[i]; }
   
  public int strength    () { return addModifs[0]; }
  public int dexterity   () { return addModifs[1]; }
  public int magic       () { return addModifs[2]; }
  public int maxLife     () { return addModifs[3]; }
  public int intelligence() { return addModifs[4]; }
  
  /** @return true if this StatModifier doesn't really modify any stats. */
  public boolean isZero ()
  {
    for (int i = 0; i < MODIFS_COUNT; i++) 
      if (addModifs[i] != 0) return false;
    return true;
  }

  /** Modify this StatModifier using given StatModifier.
     Note: here dexterity is forced to be >= 1. */
  public void apply (StatModifier sta)
  {
    for (int i=0; i < MODIFS_COUNT; i++) 
      addModifs[i] += sta.addModifs[i];
  }
  
  /** Zasadniczo modyfikatory addytywne mog mie zupenie swobodne
     wartoci, take ujemne. To ma sens i jest uyteczne, chociaby
     przeklte przedmioty na pewno bd tego uyway. Ale z drugiej
     strony kiedy StatModifier okrelaj kocowy zestaw modyfikatorw
     pewnej postaci to nakadamy na nie pewne ograniczenia,
     np. wszystkie cechy musz by > 0 (chocia zasadniczo logika gry
     wymaga w tym momencie tylko aby maxLife > 0 i dexterity > 0
     (eby Being.getDelay dziaao)).
     Ta funkcja wanie wymusza te ograniczenia. */
  public void makeFinalStat()
  {          
    for (int i=0; i < MODIFS_COUNT; i++) 
      if (addModifs[i] <= 0) addModifs[i] = 1;
  }

  /** Jeeli isZero to zwraca "" (string pusty, zerowej dugoci).
     Wpp. zwrci co w rodzaju "[" + modyfikowane cechy + "]", np.
     "[sia: -2]" */
  public String toString ()
  {
    String res;

    if (isZero()) return "";
    res = "[";
    
    for (int i=0; i < MODIFS_COUNT; i++) 
      if (addModifs[i] != 0) res += addModifsNames[i] + ": "  + addModifs[i] + " ";
      
    if (res.endsWith (" ")) 
      res = res.substring (0, res.length () - 1);
         
    res += "]";
       
    return res;
  }
}

/* 
 * $Log: StatModifier.java,v $
 * Revision 1.1  2004/02/18 14:37:34  michalis
 * * pl stringi
 * * --help output changed (to reflect that those classes will be run by bin/scripts)
 * preparations for ClientApplet:
 * * Client rozdzielony na Client i ClientStandalone
 * * znaczna cz pakietu common przeniesiona do nowego pakietu game
 * X* make jamy_i_nory_client_applet.jar
 *
 * Revision 1.21  2004/02/09 23:08:50  michalis
 * + GPL license clause i copyrighty (moje i gdzieniegdzie Michaa, sorry Rafa
 *   ale naprawd nic nie zostao) do wszystkich *.java
 *
 * Revision 1.20  2004/02/07 21:11:32  michalis
 * * StatModifier.makeFinalStat changed (bugfix: must be maxLife > 0,
 *   bez tego GrojecKiller czsto zaczyna ju jako trup)
 * * StatModifier.Modifs are named (as it was before) addModifs
 * * small improvement to Creature.action (to m.in. eliminuje problem ktry mi
 *   opisalicie e po zabiciu DungeonExplorera on jeszcze zdy (martwy)
 *   wyj z pomieszczenia)
 * + Utils.clamped
 * * Being.getDelay lepiej dostrojone
 *
 * Revision 1.19  2004/02/04 09:09:23  michalis
 * + contract of StatModifier.toString specified
 * + Location.statModifier(), odpowiednio rozszerzone Location.toString
 * * Being.fullStats odpowiednio rozszerzone
 * + lokacja Bagno (eby przetestowa Location.statModifier()),
 * * Goblin przeniesiony z tyka dinozura na zachodni balkon,
 *   Troll przeniesiony z tyka dinozura na wschodni balkon
 *   (eby ZPL nie pokazywa lokacji o nazwie "tyek dinozaura")
 *
 * Revision 1.18  2004/01/18 22:27:19  michalis
 * + StatModifier.MODIF_ consts added, * some things rewritten using these consts,
 * + StatModifier.makeFinalStat, used in Being.fullStats
 *
 * Revision 1.17  2004/01/18 17:48:02  michalis
 * * StatModifier.MODIFS_COUNT, ModifsNames made public, some minor changes
 *
 * Revision 1.16  2004/01/16 19:39:22  michalis
 * * Poprawione polskie ogonki z Windows-1250 na ISO-8859-2. Rafa: no wiesz o co chodzi.
 *
 * Revision 1.15  2004/01/15 18:33:34  michalis
 * * Small updates; strings made more consistent (only polish, with ogonki)
 *
 * Revision 1.14  2004/01/15 13:19:42  emes
 * * s/Pullic/Public/
 *
 * Revision 1.13  2004/01/15 11:21:27  michalis
 * * Fixed indentation (Rafal: nie uzywaj znakow TAB w zrodlach !)
 *
 * Revision 1.12  2004/01/15 00:41:08  bularaider
 * + public StatModifier (int[] stat),
 * + public int getModifier (int i),
 * + javadoc, few comments,
 * * small identifier changes.
 *
 * Revision 1.10  2004/01/12 02:02:44  michalis
 * + StatModifier.intelligence(), class Attitude, Being.attitude(),
 *   Being.toString(Being) extended to show attitude
 *
 * Revision 1.9  2004/01/10 09:39:47  michalis
 * * StatModifier interface and implementation changed;
 *   now it will be safer and easier to add new stats there
 *
 * Revision 1.8  2004/01/10 08:55:10  michalis
 * + StatModifier.maxLife and Being.life properties
 *
 * Revision 1.7  2004/01/10 07:21:34  michalis
 * * Finalized name adjustments (str -> strength etc.)
 *
 * Revision 1.6  2004/01/10 03:08:58  michalis
 * * Many small fixes; mainly improvements to javadoc comments
 *
 * Revision 1.5  2004/01/09 00:44:52  emes
 * - StatModifier() made public
 *
 * Revision 1.4  2004/01/08 22:45:38  michalis
 * - Usunalem wlasny blad - te "reczne" logi ktore zaczalem tworzyc
 *
 * Revision 1.3  2004/01/08 22:37:42  michalis
 * + CVS Logs appended to all .java files
 *
 *
 */
