/*
  Copyright 2004 Michalis Kamburelis.

  This file is part of "Jamy & Nory".

  "Jamy & Nory" is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  "Jamy & Nory" is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with "Jamy & Nory"; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package jamy_i_nory.server;

import java.util.*;
import jamy_i_nory.common.*;
import jamy_i_nory.common.iosystem.*;
import jamy_i_nory.game.*;

/** TODO: ta klasa BAGA aby jej kod poczy z UserCommand. Naprawd.
   Niektre jej kawaki zostay wrcz SKOPIOWANE z kodu UserCommand.
   Straszne. Ale tak byo. */
abstract class ServerCommand {

  static class CloseServerCommandException extends Exception {};
  
  /** doc: patrz UserCommandBlackTokens.commandPattern */ 
  final public String[] commandPattern;

  /** Cannot be null. */
  final public String doc;  
  
  ServerCommand(String[] aCommandPattern, String aDoc)
  {
    commandPattern = aCommandPattern;
    doc = aDoc;
  }
  
  ServerCommand(String aCommandPattern1, String aDoc)
  {
    this(new String[] { aCommandPattern1 }, aDoc);
  }
  
  /** Check does given command matches our commandPattern. */
  public boolean matches(String[] command)
  {
    return Utils.stringArraysNullEqual(command, commandPattern);
  }

  /** doc: patrz UserCommand.docLine */
  public String docLine()
  {
    StringBuffer result = new StringBuffer();
    int argsShowed = 0;

    for (int i = 0; i < commandPattern.length; i++)
      result.append( (commandPattern[i] != null ? 
        commandPattern[i] : "ARG" + (++argsShowed)) +" ");
    result.append("- " +doc);
    
    return result.toString();
  } 
  
  /** doc: patrz UserCommand.parseId, tylko e tutaj wypisuje bd na
     server.io.println */
  protected Long parseId(Server server, String s)
  {
    try {
      return Long.valueOf(s);
    } catch (NumberFormatException e) {
      server.io.println("Nieprawidowy format identyfikatora " 
        +"(liczby cakowitej): " +e.getMessage());
      return null;
    }
  }
  
  /** Parsuje id z s i szuka postaci o takim id w server.world.
     Zwraca null i wypisuje odpowiedni komunikat bdu jeeli nie znalaz
     (albo jeeli s nie reprezentuje identyfikatora, tzn. liczby cakowitej). */
  protected Being parseBeingId(Server server, String s)
  {
    Long ido = parseId(server, s);
    if (ido == null) return null;
    long id = ido.longValue();
    
    Being result = server.world.beingsFindById(id);
    if (result == null)
      server.io.println("Nie ma postaci o id = " +id +".");
      
    return result;
  }
  
  public abstract void execute(Server server, String[] command)
  throws CloseServerCommandException;
  
  /* static members ---------------------------------------- */

  final private static ServerCommand availCommands[] =
  {
    new ServerCommand("koniec", "zamknij serwer \"czysto\" (moesz take "
      +"zakoczy stdin lub zamkn okienko Swinga)") {
      public void execute(Server server, String[] command)
      throws CloseServerCommandException
      {
        throw new CloseServerCommandException();
      }
    },
    
    new ServerCommand("pomoc", "poka t pomoc") {
      public void execute(Server server, String[] command)
      {
        server.io.println("Dostpne polecenia:");
        for (int i = 0; i < availCommands.length; i++)
          server.io.println("  " +availCommands[i].docLine());
      }
    },
    
    new ServerCommand("gracze", "poka wszystkich graczy") {
      public void execute(Server server, String[] command)
      {
        server.io.print(server.world.allPlayersToString());
      }
    },
    
    new ServerCommand("stworzenia", 
      "poka wszystkie stworzenia sterowane przez komputer") {
      public void execute(Server server, String[] command)
      {
        server.io.print(server.world.allCreaturesToString());
      }
    },
    
    new ServerCommand("mrozonki", 
      "poka wszystkie mroonki (zamroone postacie graczy "+
      "ktrzy odczyli si od serwera)") {
      public void execute(Server server, String[] command)
      {
        server.io.print(Utils.listToString(server.inactivePlayers, "Mroonki"));
      }
    },
    
    new ServerCommand("przedmioty", 
      "poka wszystkie przedmioty w grze lece w lokacjach") {
      public void execute(Server server, String[] command)
      {
        int itemsCount = 0;
        for (int i = 0; i < server.world.locations.length; i++)
        {
          Location loc = server.world.locations[i];
          Iterator iter = loc.items().iterator();
          while (iter.hasNext())
          {
            Item item = (Item) iter.next();
            itemsCount++;
            server.io.println(loc.name +": " +item);
          }
        }
        server.io.println("Razem " +itemsCount +" przedmiotw.");
      }
    },
    
    new ServerCommand(new String[] { "opisz", null }, 
      "opisz dokadnie posta o id = ARG1 (TODO-w przyszoci zbir id "+
      "jakie mona tu podawa bdzie zwikszany)") {
      public void execute(Server server, String[] command)
      {
        Being b = parseBeingId(server, command[1]);
        if (b != null)
          server.io.println(b.toDetailString());
      }
    },
    
    new ServerCommand(new String[] { "zabij", null }, 
      "zabij posta o id = ARG1") {
      public void execute(Server server, String[] command)
      {
        Being b = parseBeingId(server, command[1]);
        if (b != null) {
          server.io.println("Zabijam " +b +".");
          b.life = 0;
        }
      }
    }
  };
  
  final static private int whichCommand(String[] command)
  {
    for (int i = 0; i < availCommands.length; i++)
      if (availCommands[i].matches(command)) return i;
    return -1;
  }
  
  final static public void execute(Server server, String commandStr)
  throws CloseServerCommandException
  {
    if (commandStr == null) 
      throw new CloseServerCommandException();
      
    String[] command = commandStr.split("\\s+");

    int whichCommand = whichCommand(command);
    if (whichCommand == -1)
    {
      server.io.println("Nieprawidowe polecenie \"" +commandStr +"\". "+
        "Uyj \"pomoc\" aby zobaczy list dostpnych polece.");
    } else
    {
      availCommands[whichCommand].execute(server, command);
    }
  }
}

/*
 * $Log: ServerCommand.java,v $
 * Revision 1.8  2004/02/18 14:37:37  michalis
 * * pl stringi
 * * --help output changed (to reflect that those classes will be run by bin/scripts)
 * preparations for ClientApplet:
 * * Client rozdzielony na Client i ClientStandalone
 * * znaczna cz pakietu common przeniesiona do nowego pakietu game
 * X* make jamy_i_nory_client_applet.jar
 *
 * Revision 1.7  2004/02/09 23:08:57  michalis
 * + GPL license clause i copyrighty (moje i gdzieniegdzie Michaa, sorry Rafa
 *   ale naprawd nic nie zostao) do wszystkich *.java
 *
 * Revision 1.6  2004/02/07 21:37:21  michalis
 * * malekie ulepszenia
 * * pusty plecak Beinga ktry ma name==null nie jest upuszczany na ziemi (bo po co?)
 *
 * Revision 1.5  2004/02/07 20:05:19  michalis
 * + World.beingsFindById
 * + ServerCommand.parseBeingId
 * * ServerCommand polecenia "opisz" i "zabij" zapisane adniej, poprawione pl ogonki
 * * implementacja Zabjcy z Grjca uproszczona
 * + do dokumentacji o poleceniach serwera "opisz" i "zabij"
 *
 * Revision 1.4  2004/02/07 13:03:09  michalis
 * + komenda "opisz"
 *
 * Revision 1.3  2004/02/07 12:50:33  michalis
 * + komenda "przedmioty"
 *
 * Revision 1.2  2004/02/06 04:44:42  michalis
 * + InactivePlayer
 * + dodatkowy konstruktor do Being
 * + dodatkowy konstruktor Player(World, InactivePlayer)
 * * beforeDeath rozszerzone o parametr goingInactive
 * + Being.goInactive
 * * listToString moved from World to Utils
 * + ServerCommand : polecenie "mrozonki"
 *
 * Revision 1.1  2004/02/06 01:28:38  michalis
 * * implementacja UserCommandBlackTokens.matches przeniesiona do Utils.stringArraysNullEqual
 * + ServerCommand
 * * Server.ioCheckInput uproszczone dziki temu
 * * komendy serwera na polski, konsekwentnie
 * + ServerCommand "zabij ID"
 *
 */
