/*
  Copyright 2004 Michalis Kamburelis, Micha Saaban.

  This file is part of "Jamy & Nory".

  "Jamy & Nory" is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  "Jamy & Nory" is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with "Jamy & Nory"; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

import jamy_i_nory.common.*;
import jamy_i_nory.game.*;

/** Simple tests to see some simplest things "in action". */
public class BaseTests {

  /* ------------------------------------------------------------
     Some variables useful in many tests. */  
     
  static private World world = new World();
  
  static private StatModifier stat = new StatModifier();
  static private StatModifier stat2 = new StatModifier (2, 0, 6, 0, 0);
  static private Creature.Troll troll = 
    new Creature.Troll(world, world.locations[0], null);
  static private Item merlinsCloak = new Armor(3.0f, "cloak", "Merlin's",
         null, stat, 1);  
    
  /* ---------------------------------------- */
  
  static private String testHit(DiceRandom h) {
    return "random hit from " +h +": " +h.random() + "\n";
  }
  
  static private void playerTest() 
  throws java.io.IOException
  {
/*    Player p = Player.newInteractive(world);
    System.out.println("Created player: " +p);
    System.out.println("Player full stats: " +p.fullStats ());  
    System.out.println("Player id: " +p.id);
*/    
    Player p2 = new Player(world, Race.human, "Gandalf's big&fat wife");
    System.out.println("Gandalf's wife id: " +p2.id);        
    System.out.println("Gandalf's wife location: " +p2.nowLocation().name);
  }
  
  static private void printAttack(Being a, Being b)
  {
    /* We must pass PlayersMessages to attack (but we will ignore it's value) */
    System.out.println(a.race +" atakuje " +b.race +": " +a.attack(b));
      
    System.out.println(a.toString());
    System.out.println(b.toString());  
  }
  
  static private void fightingTest()
  {
    Being b1 = new Creature.Troll(world, world.locations[0], "Old Stony Grandpa");
    Being b2 = new SimpleMindedCreature(world, world.locations[0],
      new Race("Worm", new StatModifier(2, 15, 0, 100, 2), 
        new DiceRandom(2, 4, 1), 0), "Little Innocent Robacek", 0, true, true);
    System.out.println(b1.toDetailString());
    System.out.println(b2.toDetailString());
    
    printAttack(b1, b2);
    printAttack(b2, b1);
    printAttack(b1, b2);
    printAttack(b2, b1);
  }
  
  static private void locationTest() 
  {
    Location loc = world.locations[0];
    
    //System.out.println(loc);
    
    loc.items().add(merlinsCloak);
    System.out.println(loc);
    
    //System.out.println(worldLocations[1]);
    //System.out.println(worldLocations[2]);
  }  

  static public void main(String[] args) 
  throws java.io.IOException
  {
    DiceRandom h = new DiceRandom(2, 6, 10);
    DiceRandom h2 = new DiceRandom(0, 0);
    
    System.out.println( "" +
        (new Armor(3.0f, "cloak", "blue", null, stat, 1)) +"\n"+
        merlinsCloak +"\n"+
        (new Weapon(6.0f, "sword", "short", null, stat, h)) +"\n"+
        (new Weapon(5.5f, "sword", "short", "Dickblade", stat, h)) +"\n"+
        (new Ring("calcite", stat2)) + "\n" +
        testHit(h) +
        testHit(h2)
      );
    System.out.println (troll.race + " " + troll.fullStats ());
    troll.wear(Being.USED_WEAPON, 
      new Weapon(8.0f, "Sword", "broad", null, stat2, h));
    System.out.println (troll.race + " " + troll.fullStats () +" id: " +troll.id);
    System.out.println("Troll weapon's id: " +troll.weapon().id);
    System.out.println("Some link to location id: " +
      ((LinkToLocation) world.locations[0].links().get(0)).id);

    playerTest();    
    locationTest();
    fightingTest();
  }

}

/* 
 * $Log: BaseTests.java,v $
 * Revision 1.34  2004/02/19 15:30:29  michalis
 * * Creature.attackPlayers/Creatures -> SimpleMindedCreature
 * + NonHostileCreature
 * + ElfFairy, ElfFairyWhistle
 * + wanderingFactor, teraz prawie wszystkie creatures wasaj si po wiecie
 *
 * Revision 1.33  2004/02/18 14:37:44  michalis
 * * pl stringi
 * * --help output changed (to reflect that those classes will be run by bin/scripts)
 * preparations for ClientApplet:
 * * Client rozdzielony na Client i ClientStandalone
 * * znaczna cz pakietu common przeniesiona do nowego pakietu game
 * X* make jamy_i_nory_client_applet.jar
 *
 * Revision 1.32  2004/02/09 23:09:03  michalis
 * + GPL license clause i copyrighty (moje i gdzieniegdzie Michaa, sorry Rafa
 *   ale naprawd nic nie zostao) do wszystkich *.java
 *
 * Revision 1.31  2004/02/04 14:12:26  michalis
 * * znowu zapomniaem uaktualni BaseTests. Traktujcie to jako cig dalszy
 *   ostatniego commita: odwoania do PlayersMessages usunite z BaseTests
 *   (czyli usunem to co dodaem kilka godzin temu... no ale kto mg przewidzie).
 *
 * Revision 1.30  2004/02/04 06:42:51  michalis
 * * Uaktualnienia eby si kompilowao - dodane new PlayersMessages(world)
 *
 * Revision 1.29  2004/01/19 01:51:10  michalis
 * * Troll -> Creature.Troll
 *
 * Revision 1.28  2004/01/18 19:11:52  michalis
 * + fightingTest
 *
 * Revision 1.27  2004/01/18 17:50:02  michalis
 * * DiceRandom.toString slightly improved (+ tests for it in BaseTests),
 * + Race.toDetailString (* used in JamySingle)
 *
 * Revision 1.26  2004/01/18 06:26:36  michalis
 * * HitModifier ranamed to DiceRandom (more general name, this class may
 *   be used in many other cases when we need the AkB+C kind of randomness),
 *   small changes: + constantBonus, - parameterless constructor (useless).
 *
 * Revision 1.25  2004/01/16 20:54:30  michalis
 * + Troll class, Troll tests in BaseTests, initial World contains one Troll
 *   instance. Finally we can see first Creature AI in action !
 *
 * Revision 1.24  2004/01/15 13:58:25  michalis
 * + Being.toDetailString, pokryte w Player, uzyte w UsedCommand,
 *   docs update in Creature, BaseTests corrected (name and kind are lowercase)
 *
 * Revision 1.23  2004/01/15 13:42:38  emes
 * * removed Sword & Cloak classes
 *
 * Revision 1.22  2004/01/14 18:50:07  michalis
 * * Item and all subclasses redesigned
 *
 * Revision 1.21  2004/01/13 20:37:02  michalis
 * + UserCommand "upusc" ogolna, BaseTests updated (weapon -> weapon())
 *
 * Revision 1.20  2004/01/13 13:08:41  michalis
 * * Small tests for Rafal
 *
 * Revision 1.19  2004/01/13 04:39:00  michalis
 * + JamySingle (single-user version of game), LinkToLocationList, UserCommand
 *
 * Revision 1.18  2004/01/12 04:08:14  michalis
 * + Creature.id moved to new class Ided, Ided is now the base class of
 *   Being, Item and LinkToLocation
 *
 * Revision 1.17  2004/01/12 02:02:47  michalis
 * + StatModifier.intelligence(), class Attitude, Being.attitude(),
 *   Being.toString(Being) extended to show attitude
 *
 * Revision 1.16  2004/01/12 01:22:48  michalis
 * + Location/Being.toString(Being), czyli obiekty z punktu
 *   widzenia danego Being
 *
 * Revision 1.15  2004/01/12 00:33:17  michalis
 * + Location.players(), beings()
 *
 * Revision 1.14  2004/01/11 23:51:58  michalis
 * * Location.world added and necessary changes made, some Player docs improved
 *
 * Revision 1.13  2004/01/11 22:48:47  michalis
 * * Class Monster removed, Monster.attackhuman/monster moved to
 *   Creature.attackPlayers/Creatures
 *
 * Revision 1.12  2004/01/11 09:42:32  michalis
 * * Minor changes
 *
 * Revision 1.11  2004/01/11 09:34:29  michalis
 * + Class LinkToLocation, Location.links and necessary tests and docs
 *
 * Revision 1.10  2004/01/11 06:51:58  michalis
 * + World class, Being.nowLocation
 *
 * Revision 1.9  2004/01/11 04:14:34  michalis
 * + Tests for class Location
 *
 * Revision 1.8  2004/01/10 08:55:12  michalis
 * + StatModifier.maxLife and Being.life properties
 *
 * Revision 1.7  2004/01/10 07:23:58  michalis
 * + Being, Player, Utils, test for creating new player interactively
 *
 * Revision 1.6  2004/01/10 03:09:04  michalis
 * * Many small fixes; mainly improvements to javadoc comments
 *
 * Revision 1.5  2004/01/09 01:03:47  emes
 * - tests for creatures
 *
 * Revision 1.4  2004/01/09 00:23:54  michalis
 * + generating documentation by javadoc, small BaseTests additions
 *
 * Revision 1.3  2004/01/08 22:37:44  michalis
 * + CVS Logs appended to all .java files
 *
 *
 */
